import copy
import math
from abjad.tools import durationtools
from abjad.tools import sequencetools
from abjad.tools.datastructuretools.ObjectInventory import ObjectInventory


class TimespanInventory(ObjectInventory):
    r'''Timespan inventory.

    Example 1::

        >>> timespan_inventory_1 = timespantools.TimespanInventory([
        ...     timespantools.Timespan(0, 3),
        ...     timespantools.Timespan(3, 6),
        ...     timespantools.Timespan(6, 10)])

    ::

        >>> z(timespan_inventory_1)
        timespantools.TimespanInventory([
            timespantools.Timespan(
                start_offset=durationtools.Offset(0, 1),
                stop_offset=durationtools.Offset(3, 1)
                ),
            timespantools.Timespan(
                start_offset=durationtools.Offset(3, 1),
                stop_offset=durationtools.Offset(6, 1)
                ),
            timespantools.Timespan(
                start_offset=durationtools.Offset(6, 1),
                stop_offset=durationtools.Offset(10, 1)
                )
            ])

    Example 2::

        >>> timespan_inventory_2 = timespantools.TimespanInventory([
        ...     timespantools.Timespan(0, 10),
        ...     timespantools.Timespan(3, 6),
        ...     timespantools.Timespan(15, 20)])

    ::

        >>> z(timespan_inventory_2)
        timespantools.TimespanInventory([
            timespantools.Timespan(
                start_offset=durationtools.Offset(0, 1),
                stop_offset=durationtools.Offset(10, 1)
                ),
            timespantools.Timespan(
                start_offset=durationtools.Offset(3, 1),
                stop_offset=durationtools.Offset(6, 1)
                ),
            timespantools.Timespan(
                start_offset=durationtools.Offset(15, 1),
                stop_offset=durationtools.Offset(20, 1)
                )
            ])

    Example 3. Empty timespan inventory:

    ::

        >>> timespan_inventory_3 = timespantools.TimespanInventory()

    ::

        >>> z(timespan_inventory_3)
        timespantools.TimespanInventory([])

    Operations on timespan currently work in place.
    '''

    ### SPECIAL METHODS ###

    def __and__(self, timespan):
        '''Keep material that intersects `timespan`:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 16),
            ...     timespantools.Timespan(5, 12),
            ...     timespantools.Timespan(-2, 8)])

        ::

            >>> timespan = timespantools.Timespan(5, 10)
            >>> result = timespan_inventory & timespan

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(5, 1),
                    stop_offset=durationtools.Offset(8, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(5, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(5, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    )
                ])

        Operate in place and return timespan inventory.
        '''
        new_timespans = []
        for current_timespan in self[:]:
            result = current_timespan & timespan
            new_timespans.extend(result)
        self[:] = sorted(new_timespans)
        return self

    def __sub__(self, timespan):
        '''Delete material that intersects `timespan`:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 16),
            ...     timespantools.Timespan(5, 12),
            ...     timespantools.Timespan(-2, 8)])

        ::

            >>> timespan = timespantools.Timespan(5, 10)
            >>> result = timespan_inventory - timespan

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(-2, 1),
                    stop_offset=durationtools.Offset(5, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(5, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(10, 1),
                    stop_offset=durationtools.Offset(12, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(10, 1),
                    stop_offset=durationtools.Offset(16, 1)
                    )
                ])

        Operate in place and return timespan inventory.
        '''
        new_timespans = []
        for current_timespan in self[:]:
            result = current_timespan - timespan
            new_timespans.extend(result)
        self[:] = sorted(new_timespans)
        return self

    ### PRIVATE METHODS ###

    def _get_offsets(self, expr):
        if hasattr(expr, 'start_offset') and hasattr(expr, 'stop_offset'):
            return expr.start_offset, expr.stop_offset
        elif hasattr(expr, 'timespan'):
            return expr.timespan.offsets
        else:
            raise TypeError(expr)

    def _get_timespan(self, expr):
        from abjad.tools import timespantools
        start_offset, stop_offset = self._get_offsets(expr)
        return timespantools.Timespan(start_offset, stop_offset)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def all_are_contiguous(self):
        '''True when all timespans are time-contiguous::

            >>> timespan_inventory_1.all_are_contiguous
            True

        False when timespans not time-contiguous::

            >>> timespan_inventory_2.all_are_contiguous
            False

        True when empty::

            >>> timespan_inventory_3.all_are_contiguous
            True

        Return boolean.
        '''
        if len(self) <= 1:
            return True
        last_stop_offset = self[0].stop_offset
        for timespan in self[1:]:
            if timespan.start_offset != last_stop_offset:
                return False
            last_stop_offset = timespan.stop_offset
        return True

    @property
    def all_are_well_formed(self):
        '''True when all timespans are well-formed::

            >>> timespan_inventory_1.all_are_well_formed
            True

        ::
        
            >>> timespan_inventory_2.all_are_well_formed
            True

        Also true when empty:

        ::

            >>> timespan_inventory_3.all_are_well_formed
            True

        Otherwise false.

        Return boolean.
        '''
        return all([self._get_timespan(expr).is_well_formed for expr in self])

    @property
    def axis(self):
        '''Arithmetic mean of start- and stop-offsets.

            >>> timespan_inventory_1.axis
            Offset(5, 1)

        ::

            >>> timespan_inventory_2.axis
            Offset(10, 1)

        None when empty:

        ::

            >>> timespan_inventory_3.axis is None
            True

        Return offset or none.
        '''
        if self:
            return (self.start_offset + self.stop_offset) / 2

    @property
    def duration(self):
        '''Time from start offset to stop offset::

            >>> timespan_inventory_1.duration
            Duration(10, 1)

        ::

            >>> timespan_inventory_2.duration
            Duration(20, 1)

        Zero when empty:

        ::

            >>> timespan_inventory_3.duration
            Duration(0, 1)

        Return duration.
        '''
        if self.stop_offset is not Infinity and self.start_offset is not NegativeInfinity:
            return self.stop_offset - self.start_offset
        else:
            return durationtools.Duration(0)

    @property
    def is_sorted(self):
        '''True when timespans are in time order:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10)])

        ::

            >>> timespan_inventory.is_sorted
            True

        Otherwise false:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(6, 10),
            ...     timespantools.Timespan(3, 6)])

        ::

            >>> timespan_inventory.is_sorted
            False
    
        Return boolean.
        '''
        if len(self) < 2:
            return True
        for left_timespan, right_timespan in sequencetools.iterate_sequence_pairwise_strict(self):
            if right_timespan.start_offset < left_timespan.start_offset:
                return False
            if left_timespan.start_offset == right_timespan.start_offset:
                if right_timespan.stop_offset < left_timespan.stop_offset:
                    return False
        return True

    @property
    def start_offset(self):
        '''Earliest start offset of any timespan:

        ::

            >>> timespan_inventory_1.start_offset
            Offset(0, 1)

        ::

            >>> timespan_inventory_2.start_offset
            Offset(0, 1)

        Negative infinity when empty:

        ::

            >>> timespan_inventory_3.start_offset
            NegativeInfinity
            
        Return offset or none.
        '''
        if self:
            return min([self._get_timespan(expr).start_offset for expr in self])
        else:
            return NegativeInfinity

    @property
    def stop_offset(self):
        '''Latest stop offset of any timespan:

        ::

            >>> timespan_inventory_1.stop_offset
            Offset(10, 1)

        ::

            >>> timespan_inventory_2.stop_offset
            Offset(20, 1)
            
        Infinity when empty:

        ::

            >>> timespan_inventory_3.stop_offset
            Infinity

        Return offset or none.
        '''
        if self:
            return max([self._get_timespan(expr).stop_offset for expr in self])
        else:
            return Infinity

    @property
    def timespan(self):
        '''Timespan inventory timespan:

        ::

            >>> timespan_inventory_1.timespan
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(10, 1))

        ::

            >>> timespan_inventory_2.timespan
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(20, 1))

        ::

            >>> timespan_inventory_3.timespan
            Timespan(start_offset=NegativeInfinity, stop_offset=Infinity)

        Return timespan.
        '''
        from abjad.tools import timespantools
        return timespantools.Timespan(self.start_offset, self.stop_offset)

    ### PUBLIC METHODS ###

    def compute_logical_and(self):
        '''Compute logical AND of timespans.

        Example 1:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 10)])

        ::

            >>> result = timespan_inventory.compute_logical_and()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    )
                ])

        Example 2:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 10),
            ...     timespantools.Timespan(5, 12)])

        ::

            >>> result = timespan_inventory.compute_logical_and()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(5, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    )
                ])

        Example 3:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 10),
            ...     timespantools.Timespan(5, 12),
            ...     timespantools.Timespan(-2, 8)])

        ::

            >>> result = timespan_inventory.compute_logical_and()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(5, 1),
                    stop_offset=durationtools.Offset(8, 1)
                    )
                ])

        Same as setwise intersection.

        Operate in place and return timespan inventory.
        '''
        if 1 < len(self):
            result = self[0]
            for timespan in self:
                if not timespan.intersects_timespan(result):
                    self[:] = []
                    return self
                else:
                    inventory = result & timespan
                    result = inventory[0]
            self[:] = [result]
        return self

    def compute_logical_or(self):
        '''Compute logical OR of timespans.

        Example 1:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory()

        ::

            >>> result = timespan_inventory.compute_logical_or()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([])

        Example 2:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 10)])

        ::

            >>> result = timespan_inventory.compute_logical_or()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    )
                ])

        Example 3:

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 10),
            ...     timespantools.Timespan(5, 12)])

        ::

            >>> result = timespan_inventory.compute_logical_or()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(12, 1)
                    )
                ])

        Example 4:

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 10),
            ...     timespantools.Timespan(5, 12),
            ...     timespantools.Timespan(-2, 2)])

        ::

            >>> result = timespan_inventory.compute_logical_or()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(-2, 1),
                    stop_offset=durationtools.Offset(12, 1)
                    )
                ])

        Example 5:

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(-2, 2),
            ...     timespantools.Timespan(10, 20)])

        ::

            >>> result = timespan_inventory.compute_logical_or()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(-2, 1),
                    stop_offset=durationtools.Offset(2, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(10, 1),
                    stop_offset=durationtools.Offset(20, 1)
                    )
                ])

        Operate in place and return timespan inventory.
        '''
        timespans = []
        if self:
            timespans = [self[0]]
            for timespan in self[1:]:
                if timespans[-1]._can_fuse(timespan):
                    inventory = timespans[-1] | timespan
                    timespans[-1:] = inventory[:]
                else:
                    timespans.append(timespan)
        self[:] = timespans
        return self

    def compute_logical_xor(self):
        '''Compute logical XOR of timespans.

        Example 1:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory()

        ::

            >>> result = timespan_inventory.compute_logical_xor()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([])

        Example 2:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 10)])

        ::

            >>> result = timespan_inventory.compute_logical_xor()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    )
                ])

        Example 3:

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 10),
            ...     timespantools.Timespan(5, 12)])

        ::

            >>> result = timespan_inventory.compute_logical_xor()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(5, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(10, 1),
                    stop_offset=durationtools.Offset(12, 1)
                    )
                ])

        Example 4:

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 10),
            ...     timespantools.Timespan(5, 12),
            ...     timespantools.Timespan(-2, 2)])

        ::

            >>> result = timespan_inventory.compute_logical_xor()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(-2, 1),
                    stop_offset=durationtools.Offset(0, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(2, 1),
                    stop_offset=durationtools.Offset(5, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(10, 1),
                    stop_offset=durationtools.Offset(12, 1)
                    )
                ])

        Example 5:

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(-2, 2),
            ...     timespantools.Timespan(10, 20)])

        ::

            >>> result = timespan_inventory.compute_logical_xor()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(-2, 1),
                    stop_offset=durationtools.Offset(2, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(10, 1),
                    stop_offset=durationtools.Offset(20, 1)
                    )
                ])

        Example 6:

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 10),
            ...     timespantools.Timespan(4, 8),
            ...     timespantools.Timespan(2, 6)])

        ::

            >>> result = timespan_inventory.compute_logical_xor()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(2, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(8, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    )
                ])

        Example 7:

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 10),
            ...     timespantools.Timespan(0, 10)])

        ::

            >>> result = timespan_inventory.compute_logical_xor()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([])

        Operate in place and return timespan inventory.
        '''
        all_fragments = []
        for i, timespan_1 in enumerate(self):
            timespan_1_fragments = [timespan_1]
            for j, timespan_2 in enumerate(self):
                if i == j:
                    continue
                revised_timespan_1_fragments = []
                for timespan_1_fragment in timespan_1_fragments:
                    if timespan_2.intersects_timespan(timespan_1_fragment):
                        result = timespan_1_fragment - timespan_2
                        revised_timespan_1_fragments.extend(result)
                    else:   
                        revised_timespan_1_fragments.append(timespan_1_fragment)
                timespan_1_fragments = revised_timespan_1_fragments
            all_fragments.extend(timespan_1_fragments)
        self[:] = all_fragments
        self.sort()
        return self

    def get_timespan_that_satisfies_time_relation(self, time_relation):
        r'''Get timespan that satisifies `time_relation`:

        ::

            >>> timespan_1 = timespantools.Timespan(2, 5)
            >>> time_relation = timerelationtools.timespan_2_starts_during_timespan_1(
            ...     timespan_1=timespan_1)

        ::

            >>> timespan_inventory_1.get_timespan_that_satisfies_time_relation(time_relation)
            Timespan(start_offset=Offset(3, 1), stop_offset=Offset(6, 1))

        Return timespan when timespan inventory contains exactly one timespan
        that satisfies `time_relation`.

        Raise exception when timespan inventory contains no timespan
        that satisfies `time_relation`.

        Raise exception when timespan inventory contains more than one timespan
        that satisfies `time_relation`.
        '''
        timespans = self.get_timespans_that_satisfy_time_relation(time_relation)
        if len(timespans) == 1:
            return timespans[0]
        elif 1 < len(timespans):
            raise Exception('extra timespan error.')
        else:
            raise Exception('missing timespan error.')

    def get_timespans_that_satisfy_time_relation(self, time_relation):
        r'''Get timespans that satisfy `time_relation`:

        ::

            >>> timespan_1 = timespantools.Timespan(2, 8)
            >>> time_relation = timerelationtools.timespan_2_starts_during_timespan_1(
            ...     timespan_1=timespan_1)

        ::

            >>> result = timespan_inventory_1.get_timespans_that_satisfy_time_relation(
            ...     time_relation)

        ::

            >>> z(result)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(3, 1),
                    stop_offset=durationtools.Offset(6, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(6, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    )
                ])

        Return new timespan inventory.
        '''
        from abjad.tools import timerelationtools
        result = []
        for timespan in self:
            if isinstance(time_relation, timerelationtools.TimespanTimespanTimeRelation):
                if time_relation(timespan_2=timespan):
                    result.append(timespan)
            elif isinstance(time_relation, timerelationtools.OffsetTimespanTimeRelation):
                if time_relation(timespan=timespan):
                    result.append(timespan)
            else:
                raise ValueError
        return type(self)(result)

    def has_timespan_that_satisfies_time_relation(self, time_relation):
        r'''True when timespan inventory has timespan that satisfies `time_relation`:

        ::

            >>> timespan_1 = timespantools.Timespan(2, 8)
            >>> time_relation = timerelationtools.timespan_2_starts_during_timespan_1(
            ...     timespan_1=timespan_1)

        ::

            >>> timespan_inventory_1.has_timespan_that_satisfies_time_relation(time_relation)
            True

        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(10, 20)
            >>> time_relation = timerelationtools.timespan_2_starts_during_timespan_1(
            ...     timespan_1=timespan_1)

        ::

            >>> timespan_inventory_1.has_timespan_that_satisfies_time_relation(time_relation)
            False

        Return boolean.
        '''
        return bool(self.get_timespans_that_satisfy_time_relation(time_relation))

    def reflect(self, axis=None):
        '''Reflect timespans.

        Example 1. Reflect timespans about timespan inventory axis:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10)])

        ::

            >>> result = timespan_inventory.reflect()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(4, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(4, 1),
                    stop_offset=durationtools.Offset(7, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(7, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    )
                ])

        Example 2. Reflect timespans about arbitrary axis:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10)])

        ::

            >>> result = timespan_inventory.reflect(axis=Offset(15))

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(20, 1),
                    stop_offset=durationtools.Offset(24, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(24, 1),
                    stop_offset=durationtools.Offset(27, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(27, 1),
                    stop_offset=durationtools.Offset(30, 1)
                    )
                ])

        Operate in place and return timespan inventory.
        '''
        if axis is None:
            axis = self.axis
        timespans = []
        for timespan in self:
            timespan = timespan.reflect(axis=axis)
            timespans.append(timespan)
        timespans.reverse()
        self[:] = timespans
        return self

    def remove_degenerate_timespans(self):
        '''Remove degenerate timespans:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(5, 5),
            ...     timespantools.Timespan(5, 10),
            ...     timespantools.Timespan(5, 25)])

        ::

            >>> result = timespan_inventory.remove_degenerate_timespans()

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(5, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(5, 1),
                    stop_offset=durationtools.Offset(25, 1)
                    )
                ])

        Operate in place and return timespan inventory.
        '''
        timespans = [x for x in self if x.is_well_formed]
        self[:] = timespans
        return self

    def repeat_to_stop_offset(self, stop_offset):
        '''Repeat timespans to `stop_offset`:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10)])

        ::

            >>> result = timespan_inventory.repeat_to_stop_offset(15)

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(3, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(3, 1),
                    stop_offset=durationtools.Offset(6, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(6, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(10, 1),
                    stop_offset=durationtools.Offset(13, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(13, 1),
                    stop_offset=durationtools.Offset(15, 1)
                    )
                ])

        Operate in place and return timespan inventory.
        '''
        assert self.is_sorted
        stop_offset = durationtools.Offset(stop_offset)
        assert self.stop_offset <= stop_offset
        current_timespan_index = 0
        if self:
            while self.stop_offset < stop_offset:
                current_timespan = self[current_timespan_index]
                translation = self.stop_offset - current_timespan.start_offset
                new_timespan = current_timespan.translate(translation)
                self.append(new_timespan)
                current_timespan_index += 1
            if stop_offset < self.stop_offset:
                self[-1] = self[-1].set_offsets(stop_offset=stop_offset)
        return self

    def rotate(self, count):
        '''Rotate by `count` contiguous timespans.

        Example 1. Rotate by one timespan to the left:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 4),
            ...     timespantools.Timespan(4, 10)])

        ::

            >>> result = timespan_inventory.rotate(-1)

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(1, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(1, 1),
                    stop_offset=durationtools.Offset(7, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(7, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    )
                ])

        Example 2. Rotate by one timespan to the right:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 4),
            ...     timespantools.Timespan(4, 10)])

        ::

            >>> result = timespan_inventory.rotate(1)

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(6, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(6, 1),
                    stop_offset=durationtools.Offset(9, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(9, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    )
                ])

        Operate in place and return timespan inventory.
        '''
        assert isinstance(count, int)
        assert self.all_are_contiguous
        elements_to_move = count % len(self)
        if elements_to_move == 0:
            return
        left_timespans = self[:-elements_to_move]
        right_timespans = self[-elements_to_move:]
        split_offset = right_timespans[0].start_offset
        translation_to_left = split_offset - self.start_offset
        translation_to_left *= -1
        translation_to_right = self.stop_offset - split_offset
        translated_right_timespans = []
        for right_timespan in right_timespans:
            translated_right_timespan = right_timespan.translate_offsets(
                translation_to_left, translation_to_left)
            translated_right_timespans.append(translated_right_timespan)
        translated_left_timespans = []
        for left_timespan in left_timespans:
            translated_left_timespan = left_timespan.translate_offsets(
                translation_to_right, translation_to_right)
            translated_left_timespans.append(translated_left_timespan)
        new_timespans = translated_right_timespans + translated_left_timespans
        self[:] = new_timespans
        return self

    def scale(self, multiplier, anchor=Left):
        '''Scale timespan by `multiplier` relative to `anchor`. 

        Example 1. Scale timespans relative to timespan inventory start offset:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10)])

        ::

            >>> result = timespan_inventory.scale(2)

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(6, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(3, 1),
                    stop_offset=durationtools.Offset(9, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(6, 1),
                    stop_offset=durationtools.Offset(14, 1)
                    )
                ])

        Example 2. Scale timespans relative to timespan inventory stop offset:

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10)])

        ::

            >>> result = timespan_inventory.scale(2, anchor=Right)

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(-3, 1),
                    stop_offset=durationtools.Offset(3, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(6, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(2, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    )
                ])

        Operate in place and return timespan inventory.
        '''
        timespans = []
        for timespan in self:
            timespan = timespan.scale(multiplier, anchor=anchor)
            timespans.append(timespan)
        self[:] = timespans
        return self

    def stretch(self, multiplier, anchor=None):
        '''Stretch timespans by `multiplier` relative to `anchor`.

        Example 1: Stretch timespans relative to timespan inventory start offset:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10)])

        ::

            >>> result = timespan_inventory.stretch(2)

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(6, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(6, 1),
                    stop_offset=durationtools.Offset(12, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(12, 1),
                    stop_offset=durationtools.Offset(20, 1)
                    )
                ])

        Example 2: Stretch timespans relative to arbitrary anchor:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10)])


        ::

            >>> result = timespan_inventory.stretch(2, anchor=Offset(8))

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(-8, 1),
                    stop_offset=durationtools.Offset(-2, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(-2, 1),
                    stop_offset=durationtools.Offset(4, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(4, 1),
                    stop_offset=durationtools.Offset(12, 1)
                    )
                ])

        Operate in place and return timespan inventory.
        '''
        timespans = []
        if anchor is None:
            anchor = self.start_offset
        for timespan in self:
            timespan = timespan.stretch(multiplier, anchor)
            timespans.append(timespan)
        self[:] = timespans
        return self

    def translate(self, translation=None):
        '''Translate timespans by `translation`.

        Example 1. Translate timespan by offset ``50``:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10)])

        ::

            >>> result = timespan_inventory.translate(50)

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(50, 1),
                    stop_offset=durationtools.Offset(53, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(53, 1),
                    stop_offset=durationtools.Offset(56, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(56, 1),
                    stop_offset=durationtools.Offset(60, 1)
                    )
                ])

        Operate in place and return timespan inventory.
        '''
        return self.translate_offsets(translation, translation)

    def translate_offsets(self, start_offset_translation=None, stop_offset_translation=None):
        '''Translate timespans by `start_offset_translation`
        and `stop_offset_translation`.

        Example 1. Translate timespan start- and stop-offsets equally::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10)])

        ::

            >>> result = timespan_inventory.translate_offsets(50, 50)

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(50, 1),
                    stop_offset=durationtools.Offset(53, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(53, 1),
                    stop_offset=durationtools.Offset(56, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(56, 1),
                    stop_offset=durationtools.Offset(60, 1)
                    )
                ])

        Example 2. Translate timespan stop-offsets only:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10)])

        ::

            >>> result = timespan_inventory.translate_offsets(stop_offset_translation=20)

        ::

            >>> z(timespan_inventory)
            timespantools.TimespanInventory([
                timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(23, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(3, 1),
                    stop_offset=durationtools.Offset(26, 1)
                    ),
                timespantools.Timespan(
                    start_offset=durationtools.Offset(6, 1),
                    stop_offset=durationtools.Offset(30, 1)
                    )
                ])

        Operate in place and return timespan inventory.
        '''
        timespans = []
        for timespan in self:
            timespan = timespan.translate_offsets(start_offset_translation, stop_offset_translation)
            timespans.append(timespan)
        self[:] = timespans
        return self



from abjad.tools import componenttools


def all_are_containers(expr):
    '''.. versionadded:: 2.6

    True when `expr` is a sequence of Abjad containers::

        >>> containers = 3 * Container("c'8 d'8 e'8")

    ::

        >>> containertools.all_are_containers(containers)
        True

    True when `expr` is an empty sequence::

        >>> containertools.all_are_containers([])
        True

    Otherwise false::

        >>> containertools.all_are_containers('foo')
        False

    Return boolean.

    Function wraps ``componenttools.all_are_components()``.
    '''
    from abjad.tools import containertools

    return componenttools.all_are_components(expr, klasses=(containertools.Container,))



from abjad.tools import componenttools
from abjad.tools import durationtools
from abjad.tools import mathtools


def remove_leaf_and_shrink_durated_parent_containers(leaf):
    r'''.. versionadded:: 1.1

    Remove `leaf` and shrink durated parent containers::

        >>> measure = Measure((4, 8), [])
        >>> measure.append(tuplettools.FixedDurationTuplet((2, 8), "c'8 d'8 e'8"))
        >>> measure.append(tuplettools.FixedDurationTuplet((2, 8), "f'8 g'8 a'8"))
        >>> beamtools.BeamSpanner(measure.leaves)
        BeamSpanner(c'8, d'8, e'8, f'8, g'8, a'8)

    ::

        >>> f(measure)
        {
            \time 4/8
            \times 2/3 {
                c'8 [
                d'8
                e'8
            }
            \times 2/3 {
                f'8
                g'8
                a'8 ]
            }
        }

    ::

        >>> leaftools.remove_leaf_and_shrink_durated_parent_containers(measure.leaves[0])

    ::

        >>> f(measure)
        {
            \time 5/12
            \scaleDurations #'(2 . 3) {
                {
                    d'8 [
                    e'8
                }
                {
                    f'8
                    g'8
                    a'8 ]
                }
            }
        }

    Return none.
    '''
    from abjad.tools import contexttools
    from abjad.tools import measuretools
    from abjad.tools import tuplettools

    prolated_leaf_duration = leaf.prolated_duration
    prolations = leaf._prolations
    current_prolation, i = durationtools.Duration(1), 0
    parent = leaf._parent

    while parent is not None and not parent.is_parallel:
        current_prolation *= prolations[i]
        if isinstance(parent, tuplettools.FixedDurationTuplet):
            candidate_new_parent_dur = parent.target_duration - current_prolation * leaf.written_duration
            if durationtools.Duration(0) < candidate_new_parent_dur:
                parent.target_duration = candidate_new_parent_dur
        elif isinstance(parent, measuretools.Measure):
            parent_time_signature = contexttools.get_time_signature_mark_attached_to_component(parent)
            old_prolation = parent_time_signature.implied_prolation
            naive_time_signature = parent_time_signature.duration - prolated_leaf_duration
            better_time_signature = mathtools.NonreducedFraction(naive_time_signature)
            better_time_signature = better_time_signature.with_denominator(parent_time_signature.denominator)
            better_time_signature = contexttools.TimeSignatureMark(better_time_signature)
            contexttools.detach_time_signature_marks_attached_to_component(parent)
            better_time_signature.attach(parent)
            parent_time_signature = contexttools.get_time_signature_mark_attached_to_component(parent)
            new_denominator = parent_time_signature.denominator
            new_prolation = parent_time_signature.implied_prolation
            adjusted_prolation = old_prolation / new_prolation
            for x in parent:
                if isinstance(x, tuplettools.FixedDurationTuplet):
                    x.target_duration *= adjusted_prolation
                else:
                    if adjusted_prolation != 1:
                        new_target = x.preprolated_duration * adjusted_prolation
                        tuplettools.FixedDurationTuplet(new_target, [x])
        parent = parent._parent
        i += 1
    parentage = componenttools.get_proper_parentage_of_component(leaf)
    componenttools.remove_component_subtree_from_score_and_spanners([leaf])
    for x in parentage:
        if not len(x):
            componenttools.remove_component_subtree_from_score_and_spanners([x])
        else:
            break



from abjad.tools import configurationtools
from abjad.tools import iotools
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript
import os


class SvnMessageScript(DeveloperScript):
    '''Edit a temporary `svn` commit message, stored in the `.abjad` directory:

    ::

        bash$ ajv svn msg -h
        usage: svn-message [-h] [--version] [-C]

        Write commit message for future commit usage.

        optional arguments:
          -h, --help   show this help message and exit
          --version    show program's version number and exit
          -C, --clean  delete previous commit message before editing

    Return `SvnMessageScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'msg'

    @property
    def commit_message_path(self):
        from abjad import ABJCFG
        commit_file = 'abjad_commit.txt'
        commit_path = os.path.join(ABJCFG.ABJAD_CONFIG_DIRECTORY_PATH, commit_file)
        return commit_path

    @property
    def long_description(self):
        return None

    @property
    def scripting_group(self):
        return 'svn'

    @property
    def short_description(self):
        return 'Write commit message for future commit usage.'

    @property
    def version(self):
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):
        text_editor = configurationtools.get_text_editor()
        if args.clean:
            if os.path.exists(self.commit_message_path):
                os.remove(self.commit_message_path)
        command = '{} {}'.format(text_editor, self.commit_message_path)
        iotools.spawn_subprocess(command)

    def setup_argument_parser(self, parser):
        parser.add_argument('-C', '--clean',
            action='store_true',
            help='delete previous commit message before editing'
            )



from abjad.tools import containertools
from abjad.tools import durationtools
from abjad.tools import mathtools


def multiply_contents_of_measures_in_expr(expr, n):
    r'''.. versionadded:: 1.1

    Multiply contents ``n - 1`` times and adjust time signature of every measure in `expr`::

        >>> measure = Measure((3, 8), "c'8 d'8 e'8")
        >>> beamtools.BeamSpanner(measure.leaves)
        BeamSpanner(c'8, d'8, e'8)

    ::

        >>> f(measure)
        {
            \time 3/8
            c'8 [
            d'8
            e'8 ]
        }

    ::

        >>> measuretools.multiply_contents_of_measures_in_expr(measure, 3)

    ::

        >>> f(measure)
        {
            \time 9/8
            c'8 [
            d'8
            e'8 ]
            c'8 [
            d'8
            e'8 ]
            c'8 [
            d'8
            e'8 ]
        }

    .. versionchanged:: 2.0
        renamed ``measuretools.spin()`` to
        ``measuretools.multiply_contents_of_measures_in_expr()``.
    '''
    from abjad.tools import contexttools
    from abjad.tools import iterationtools

    assert isinstance(n, int)
    assert 0 < n

    for measure in iterationtools.iterate_measures_in_expr(expr):
        old_time_signature = contexttools.get_effective_time_signature(measure)
        containertools.repeat_contents_of_container(measure, n)
        old_pair = (old_time_signature.numerator, old_time_signature.denominator)
        new_pair = mathtools.NonreducedFraction(old_pair)
        new_pair = new_pair.multiply_without_reducing(n)
        time_signature = contexttools.TimeSignatureMark(new_pair)
        contexttools.detach_time_signature_marks_attached_to_component(measure)
        time_signature.attach(measure)



from abjad.tools.abctools.ImmutableAbjadObject import ImmutableAbjadObject
from fractions import Fraction


class NonreducedFraction(ImmutableAbjadObject, Fraction):
    r'''.. versionadded:: 2.9

    Initialize with an integer numerator and integer denominator::
        
        >>> mathtools.NonreducedFraction(3, 6)
        NonreducedFraction(3, 6)

    Initialize with only an integer denominator::

        >>> mathtools.NonreducedFraction(3)
        NonreducedFraction(3, 1)

    Initialize with an integer pair::

        >>> mathtools.NonreducedFraction((3, 6))
        NonreducedFraction(3, 6)

    Initialize with an integer singleton::

        >>> mathtools.NonreducedFraction((3,))
        NonreducedFraction(3, 1)
        
    Similar to built-in fraction except that numerator and denominator do not reduce.

    Nonreduced fractions inherit from built-in fraction::

        >>> isinstance(mathtools.NonreducedFraction(3, 6), Fraction)
        True

    Nonreduced fractions are numbers::

        >>> import numbers

    ::

        >>> isinstance(mathtools.NonreducedFraction(3, 6), numbers.Number)
        True
    
    Nonreduced fractions are immutable.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_numerator', '_denominator')

    ### INITIALIZER ###

    def __new__(klass, *args):
        from abjad.tools import mathtools
        from abjad.tools import sequencetools
        if len(args) == 1 and hasattr(args[0], 'numerator') and hasattr(args[0], 'denominator'):
            numerator = args[0].numerator
            denominator = args[0].denominator
        elif len(args) == 1 and isinstance(args[0], int):
            numerator = args[0]
            denominator = 1
        elif len(args) == 1 and sequencetools.is_integer_singleton(args[0]):
            numerator = args[0][0]
            denominator = 1
        elif len(args) == 1 and sequencetools.is_integer_pair(args[0]):
            numerator, denominator = args[0]
        elif sequencetools.is_integer_pair(args):
            numerator = args[0]
            denominator = args[1]
        else:
            raise ValueError('can not initialize NonreducedFraction from {!r}.'.format(args))
        numerator *= mathtools.sign(denominator)
        denominator = abs(denominator)
        self = Fraction.__new__(klass, numerator, denominator)
        self._numerator = numerator
        self._denominator = denominator
        return self

    ### SPECIAL METHODS ###

    def __abs__(self):
        '''Absolute value of nonreduced fraction::

            >>> abs(mathtools.NonreducedFraction(-3, 3))
            NonreducedFraction(3, 3)

        Return nonreduced fraction.
        '''
        return type(self)((abs(self.numerator), self.denominator))
        
    def __add__(self, expr):
        '''Add `expr` to `self`::

            >>> mathtools.NonreducedFraction(3, 3) + 1
            NonreducedFraction(6, 3)

        Return nonreduced fraction.
        '''
        denominators = [self.denominator]
        if isinstance(expr, type(self)):
            denominators.append(expr.denominator) 
            expr = expr.reduce()
        fraction = self.reduce() + expr
        return self._fraction_with_denominator(fraction, max(denominators))

    def __div__(self, expr):
        '''Divide nonreduced fraction by expr::

            >>> mathtools.NonreducedFraction(3, 3) / 1
            NonreducedFraction(3, 3)

        Return nonreduced fraction.
        '''
        denominators = [self.denominator]
        if isinstance(expr, type(self)):
            denominators.append(expr.denominator)
            expr = expr.reduce()
        fraction = self.reduce() / expr
        return self._fraction_with_denominator(fraction, max(denominators))

    def __eq__(self, expr):
        '''True when `expr` equals `self`::

            >>> mathtools.NonreducedFraction(3, 3) == 1
            True
    
        Return boolean.
        '''
        return self.reduce() == expr

    def __ge__(self, expr):
        '''True when nonreduced fraction is greater than or equal to `expr`::

            >>> mathtools.NonreducedFraction(3, 3) >= 1
            True
            
        Return boolean.
        '''
        return self.reduce() >= expr

    def __gt__(self, expr):
        '''True when nonreduced fraction is greater than `expr`::

            >>> mathtools.NonreducedFraction(3, 3) > 1
            False
            
        Return boolean.
        '''
        return self.reduce() > expr

    def __le__(self, expr):
        '''True when nonreduced fraction is less than or equal to `expr`::

            >>> mathtools.NonreducedFraction(3, 3) <= 1
            True
            
        Return boolean.
        '''
        return self.reduce() <= expr

    def __lt__(self, expr):
        '''True when nonreduced fraction is less than `expr`::

            >>> mathtools.NonreducedFraction(3, 3) < 1
            False

        Return boolean.
        '''
        return self.reduce() < expr

    def __mul__(self, expr):
        '''Multiply nonreduced fraction by expr::

            >>> mathtools.NonreducedFraction(3, 3) * 3
            NonreducedFraction(9, 3)

        Return nonreduced fraction.
        '''
        denominators = [self.denominator]
        if isinstance(expr, type(self)):
            denominators.append(expr.denominator)
            expr = expr.reduce()
        fraction = self.reduce() * expr
        return self._fraction_with_denominator(fraction, max(denominators))

    def __ne__(self, expr):
        '''True when `expr` does not equal `self`::
            
            >>> mathtools.NonreducedFraction(3, 3) != 'foo'
            True

        Return boolean.
        '''
        return not self == expr

    def __neg__(self):
        '''Negate nonreduced fraction::

            >>> -mathtools.NonreducedFraction(3, 3)
            NonreducedFraction(-3, 3)

        Return nonreduced fraction.
        '''
        return type(self)((-self.numerator, self.denominator))

    def __radd__(self, expr):
        '''Add nonreduced fraction to `expr`::

            >>> 1 + mathtools.NonreducedFraction(3, 3)
            NonreducedFraction(6, 3)

        Return nonreduced fraction.
        '''
        return self + expr

    def __rdiv__(self, expr):
        '''Divide `expr` by nonreduced fraction::

            >>> 1 / mathtools.NonreducedFraction(3, 3)
            NonreducedFraction(3, 3)

        Return nonreduced fraction.
        '''
        denominators = [self.denominator]
        if isinstance(expr, type(self)):
            denominators.append(expr.denominator)
            expr = expr.reduce()
        fraction = expr / self.reduce()
        return self._fraction_with_denominator(fraction, max(denominators))

    def __rmul__(self, expr):
        '''Multiply `expr` by nonreduced fraction::

            >>> 3 * mathtools.NonreducedFraction(3, 3)
            NonreducedFraction(9, 3)

        Return nonreduced fraction.
        '''
        return self * expr

    def __rsub__(self, expr):
        '''Subtract nonreduced fraction from `expr`::

            >>> 1 - mathtools.NonreducedFraction(3, 3)
            NonreducedFraction(0, 3)

        Return nonreduced fraction.
        '''
        return -self + expr

    def __str__(self):
        '''String representation of nonreduced fraction::

            >>> fraction = mathtools.NonreducedFraction(-6, 3)
    
        ::

            >>> str(fraction)
            '-6/3'
        
        Return string.
        '''
        return '{}/{}'.format(self.numerator, self.denominator)

    def __sub__(self, expr):
        '''Subtract `expr` from self::

            >>> mathtools.NonreducedFraction(3, 3) - 2
            NonreducedFraction(-3, 3)

        Return nonreduced fraction.
        '''
        denominators = [self.denominator]
        if isinstance(expr, type(self)):
            denominators.append(expr.denominator) 
            expr = expr.reduce()
        fraction = self.reduce() - expr
        return self._fraction_with_denominator(fraction, max(denominators))

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _positional_argument_values(self):
        return self.numerator, self.denominator

    ### PRIVATE METHODS ###

    def _fraction_with_denominator(self, fraction, denominator):
        from abjad.tools import durationtools
        from abjad.tools import mathtools
        denominator = mathtools.least_common_multiple(denominator, fraction.denominator)
        return NonreducedFraction(fraction).with_denominator(denominator)
        
    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def denominator(self):
        '''Read-only denominator of nonreduced fraction::

            >>> fraction = mathtools.NonreducedFraction(-6, 3)

        ::

            >>> fraction.denominator
            3

        Return positive integer.
        '''
        return self._denominator

    @property
    def numerator(self):
        '''Read-only numerator of nonreduced fraction::

            >>> fraction = mathtools.NonreducedFraction(-6, 3)

        ::

            >>> fraction.numerator
            -6

        Return integer.
        '''
        return self._numerator

    @property
    def pair(self):
        '''Read only pair of nonreduced fraction numerator and denominator::

            >>> fraction = mathtools.NonreducedFraction(-6, 3)

        ::

            >>> fraction.pair
            (-6, 3)

        Return integer pair.
        '''
        return self.numerator, self.denominator

    ### PUBLIC METHODS ###

    def multiply_with_cross_cancelation(self, multiplier):
        '''.. versionadded:: 2.11

        Multiply nonreduced fraction by `expr` with cross-cancelation::

            >>> mathtools.NonreducedFraction(4, 8).multiply_with_cross_cancelation((2, 3))
            NonreducedFraction(4, 12)

        ::

            >>> mathtools.NonreducedFraction(4, 8).multiply_with_cross_cancelation((4, 1))
            NonreducedFraction(4, 2)

        ::

            >>> mathtools.NonreducedFraction(4, 8).multiply_with_cross_cancelation((3, 5))
            NonreducedFraction(12, 40)

        ::

            >>> mathtools.NonreducedFraction(4, 8).multiply_with_cross_cancelation((6, 5))
            NonreducedFraction(12, 20)

        ::

            >>> mathtools.NonreducedFraction(5, 6).multiply_with_cross_cancelation((6, 5))
            NonreducedFraction(1, 1)

        Return nonreduced fraction.
        '''
        from abjad.tools import durationtools
        from abjad.tools import mathtools

        multiplier = durationtools.Multiplier(multiplier)

        self_numerator_factors = mathtools.factors(self.numerator)
        multiplier_denominator_factors = mathtools.factors(multiplier.denominator)
        for factor in multiplier_denominator_factors[:]:
            if factor in self_numerator_factors:
                self_numerator_factors.remove(factor)
                multiplier_denominator_factors.remove(factor)

        self_denominator_factors = mathtools.factors(self.denominator)
        multiplier_numerator_factors = mathtools.factors(multiplier.numerator)
        for factor in multiplier_numerator_factors[:]:
            if factor in self_denominator_factors:
                self_denominator_factors.remove(factor)
                multiplier_numerator_factors.remove(factor)

        result_numerator_factors = self_numerator_factors + \
            multiplier_numerator_factors
        result_denominator_factors = self_denominator_factors + \
            multiplier_denominator_factors

        result_numerator = 1
        for factor in result_numerator_factors:
            result_numerator *= factor

        result_denominator = 1
        for factor in result_denominator_factors:
            result_denominator *= factor

        return NonreducedFraction(result_numerator, result_denominator)

    def multiply_with_numerator_preservation(self, multiplier):
        '''.. versionadded:: 2.11

        Multiply nonreduced fraction by `multiplier` with numerator preservation 
        where possible.

            >>> mathtools.NonreducedFraction(9, 16).multiply_with_numerator_preservation((2, 3))
            NonreducedFraction(9, 24)

        ::

            >>> mathtools.NonreducedFraction(9, 16).multiply_with_numerator_preservation((1, 2))
            NonreducedFraction(9, 32)

        ::

            >>> mathtools.NonreducedFraction(9, 16).multiply_with_numerator_preservation((5, 6))
            NonreducedFraction(45, 96)

        ::

            >>> mathtools.NonreducedFraction(3, 8).multiply_with_numerator_preservation((2, 3))
            NonreducedFraction(3, 12)

        Return nonreduced fraction.
        '''
        from abjad.tools import durationtools

        multiplier = durationtools.Multiplier(multiplier)
        self_denominator = self.denominator
        candidate_result_denominator = self_denominator / multiplier

        if candidate_result_denominator.denominator == 1:
            return NonreducedFraction(self.numerator, candidate_result_denominator.numerator)
        else:
            result_numerator = self.numerator * candidate_result_denominator.denominator
            result_denominator = candidate_result_denominator.numerator
            return NonreducedFraction(result_numerator, result_denominator)

    def multiply_without_reducing(self, expr):
        '''.. versionadded:: 2.11

        Multiply nonreduced fraction by `expr` without reducing::

            >>> mathtools.NonreducedFraction(3, 8).multiply_without_reducing((3, 3))
            NonreducedFraction(9, 24)

        ::

            >>> mathtools.NonreducedFraction(4, 8).multiply_without_reducing((4, 5))
            NonreducedFraction(16, 40)

        ::

            >>> mathtools.NonreducedFraction(4, 8).multiply_without_reducing((3, 4))
            NonreducedFraction(12, 32)

        Return nonreduced fraction.
        '''
        expr = NonreducedFraction(expr)
        numerator = self.numerator * expr.numerator
        denominator = self.denominator * expr.denominator
        return NonreducedFraction(numerator, denominator)

    def reduce(self):
        '''Reduce nonreduced fraction::

            >>> fraction = mathtools.NonreducedFraction(-6, 3)

        ::

            >>> fraction.reduce()
            Fraction(-2, 1)
            
        Return fraction.
        '''
        return Fraction(self.numerator, self.denominator)

    def with_denominator(self, denominator):
        '''Return new nonreduced fraction with integer `denominator`.

            >>> mathtools.NonreducedFraction(3, 6).with_denominator(12)
            NonreducedFraction(6, 12)

        Return nonreduced fraction.
        '''
        from abjad.tools import durationtools
        n, d = self.pair
        multiplier = durationtools.Multiplier(denominator, d)
        new_numerator = multiplier * n
        new_denominator = multiplier * d
        if new_numerator.denominator == 1 and new_denominator.denominator == 1:
            return type(self)(new_numerator.numerator, new_denominator.numerator)
        else:
            return type(self)(n, d)

    def with_multiple_of_denominator(self, denominator):
        '''Return new nonreduced fraction with multiple of integer `denominator`.

            >>> mathtools.NonreducedFraction(3, 6).with_multiple_of_denominator(5)
            NonreducedFraction(5, 10)

        Return nonreduced fraction.
        '''
        result = self.with_denominator(denominator)
        while not result.denominator == denominator:
            denominator *= 2
            result = result.with_denominator(denominator)

        return result



from abjad import *


def test_spannertools_iterate_components_in_spanner_01():

    t = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    p = beamtools.BeamSpanner(t[:])

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [
            d'8
        }
        {
            \time 2/8
            e'8
            f'8 ]
        }
    }
    '''

    components = spannertools.iterate_components_in_spanner(p, reverse=True)
    components = list(components)
    leaves = t.leaves

    assert components[0] is t[-1]
    assert components[1] is leaves[-1]
    assert components[2] is leaves[-2]
    assert components[3] is t[-2]
    assert components[4] is leaves[-3]
    assert components[5] is leaves[-4]


def test_spannertools_iterate_components_in_spanner_02():

    t = Staff("c'8 d'8 e'8 f'8")
    spanner = beamtools.BeamSpanner(t[2:])

    notes = spannertools.iterate_components_in_spanner(spanner, klass=Note)
    assert list(notes) == t[2:]



from abjad.tools import durationtools
from abjad.tools import skiptools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker
import fractions


class SkipRhythmMaker(RhythmMaker):
    r'''.. versionadded:: 2.10

    Skip rhythm-maker::

        >>> maker = rhythmmakertools.SkipRhythmMaker()

    ::

        >>> divisions = [(1, 5), (1, 4), (1, 6), (7, 9)]
        >>> leaf_lists = maker(divisions)
        >>> leaves = sequencetools.flatten_sequence(leaf_lists)

    ::

        >>> staff = Staff(leaves)

    ::

        >>> f(staff)
        \new Staff {
            s1 * 1/5
            s1 * 1/4
            s1 * 1/6
            s1 * 7/9
        }

    Usage follows the two-step instantiate-then-call pattern shown here.

    Return rhythm-maker.
    '''

    ### CLASS ATTRIBUTES ###

    _default_positional_input_arguments = ()

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        result = []
        for division in divisions:
            written_duration = durationtools.Duration(1)
            multiplied_duration = division
            skip = skiptools.make_skips_with_multiplied_durations(written_duration, [multiplied_duration])
            result.append(skip)
        return result



from abjad import *


def test_componenttools_get_proper_contents_of_component_01():

    staff = Staff("c' d' e' f'")

    assert componenttools.get_proper_contents_of_component(staff) == staff[:]


def test_componenttools_get_proper_contents_of_component_02():

    note = Note("c'4")

    assert componenttools.get_proper_contents_of_component(note) == []



def get_lilypond_comment_attached_to_component(component):
    r'''.. versionadded:: 2.0

    Get exactly one LilyPond comment attached to `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> marktools.LilyPondComment('comment')(staff[0])
        LilyPondComment('comment')(c'8)

    ::

        >>> f(staff)
        \new Staff {
            % comment
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> marktools.get_lilypond_comment_attached_to_component(staff[0])
        LilyPondComment('comment')(c'8)

    Return one LilyPond comment.

    Raise missing mark error when no LilyPond comment is attached.

    Raise extra mark error when more than one LilyPond comment is attached.
    '''
    from abjad.tools import marktools

    lilypond_comments = marktools.get_lilypond_comments_attached_to_component(component)
    if not lilypond_comments:
        raise MissingMarkError
    elif 1 < len(lilypond_comments):
        raise ExtraMarkError
    else:
        return lilypond_comments[0]



'''Page layout tools.

   This package depends on the following:

      * rational numbers
'''

from abjad.tools import importtools

importtools.import_structured_package(__path__[0], globals(), package_root_name='abjad')

_documentation_section = 'core'



from get_context_mark_format_pieces import get_context_mark_format_pieces
from is_formattable_context_mark_for_component import is_formattable_context_mark_for_component
import inspect


def get_all_mark_format_contributions(component):
    '''Get all mark format contributions as nested dictionaries.

    The first level of keys represent format slots.

    The second level of keys represent format contributor ('articulations', 'markup', etc.).

    Return dict.
    '''

    from abjad.tools import componenttools
    from abjad.tools import contexttools
    from abjad.tools import marktools
    from abjad.tools import markuptools

    class_to_section = {
        marktools.Articulation:        ('articulations', False),
        marktools.BendAfter:           ('articulations', False),
        marktools.LilyPondCommandMark: ('lilypond command marks', False),
        marktools.LilyPondComment:     ('comments', False),
        marktools.StemTremolo:         ('stem tremolos', True),
    }

    contributions = {}

    marks = marktools.get_marks_attached_to_component(component)

    up_markup, down_markup, neutral_markup = [], [], []

    context_marks = []

    ### organize marks attached directly to component ###

    for mark in marks:

        ### non-printing marks are skipped (i.e. Annotation) ###

        if not hasattr(mark, 'lilypond_format'):
            continue

        ### a recognized mark class ###

        section, singleton = None, False
        if mark.__class__ in class_to_section:
            section, singleton = class_to_section[mark.__class__]

        ### context marks to be dealt with later ###

        elif isinstance(mark, contexttools.ContextMark):
            if is_formattable_context_mark_for_component(mark, component):
                context_marks.append(mark)
                continue

        ### markup to be dealt with later ###

        elif isinstance(mark, markuptools.Markup):
            if mark.direction is Up:
                up_markup.append(mark)
            elif mark.direction is Down:
                down_markup.append(mark)
            elif mark.direction in (Center, None):
                neutral_markup.append(mark)
            continue

        ### otherwise, test if mark is a subclass of a recognized mark ###

        else:
            mro = list(inspect.getmro(mark.__class__))
            while mro:
                if mro[-1] in class_to_section:
                    section, singleton = class_to_section[mro[-1]]
                mro.pop()
            if not section:
                section, singleton = 'other marks', False

        ### prepare the contributions dictionary ###

        format_slot = mark._format_slot
        if format_slot not in contributions:
            contributions[format_slot] = { }
        if section not in contributions[format_slot]:
            contributions[format_slot][section] = []

        ### add the mark contribution ###

        contribution_list = contributions[format_slot][section]
        if len(contribution_list) and singleton:
            raise ExtraMarkError
        result = mark.lilypond_format
        assert isinstance(result, str)
        contribution_list.append(result)
        if section == 'articulations':
            contribution_list.sort()

    ### handle context marks ###

    for parent in componenttools.get_proper_parentage_of_component(component):
        for mark in parent._marks_for_which_component_functions_as_start_component:
            if isinstance(mark, contexttools.ContextMark):
                if mark not in context_marks:
                    if is_formattable_context_mark_for_component(mark, component):
                        context_marks.append(mark)

    #for candidate in context_mark_candidates:
    #    if candidate not in context_marks:
    #            context_marks.append(candidate)

    section = 'context marks'
    for mark in context_marks:
        format_slot = mark._format_slot
        result = get_context_mark_format_pieces(mark)
        if format_slot not in contributions:
            contributions[format_slot] = { }
        if section not in contributions[format_slot]:
            contributions[format_slot][section] = []
        contributions[format_slot][section].extend(result)

    ### handle markup ###

    result = []
    for markup_list in (up_markup, down_markup, neutral_markup):
        if not markup_list:
            pass
        elif 1 < len(markup_list):
            contents = []
            for m in markup_list:
                contents += m.contents
            direction = markup_list[0].direction
            if direction is None:
                direction = '-'
            command = markuptools.MarkupCommand('column', contents)
            markup = markuptools.Markup(command, direction=direction)
            result.extend(markup._get_format_pieces(is_indented=True))
        else:
            if markup_list[0].direction is None:
                markup = markuptools.Markup(markup_list[0])
                markup.direction = '-'
                result.extend(markup._get_format_pieces(is_indented=True))
            else:
                result.extend(markup_list[0]._get_format_pieces(is_indented=True))

    if result:
        if 'right' not in contributions:
            contributions['right'] = {}
        contributions['right']['markup'] = result

    for slot in contributions:
        for kind, value in contributions[slot].iteritems():
            contributions[slot][kind] = tuple(value)

    return contributions



from abjad import *
from abjad.tools import rhythmtreetools


def test_RhythmTreeNode___call___01():

    rtm = '(1 (1 1 1 1))'
    tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
    result = tree((1, 4))

    assert isinstance(result, (list, selectiontools.Selection))
    assert len(result) == 4
    assert all([isinstance(x, Note) for x in result])
    assert all([x.written_duration == Duration(1, 16) for x in result])


def test_RhythmTreeNode___call___02():

    rtm = '(1 (1 (2 (1 1 1)) 2))'
    tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
    result = tree((1, 4))

    assert isinstance(result, (list, selectiontools.Selection))
    assert len(result) == 1
    assert isinstance(result[0], tuplettools.FixedDurationTuplet)
    assert result[0].lilypond_format == "\\times 4/5 {\n\tc'16\n\t\\times 2/3 {\n\t\tc'16\n\t\tc'16\n\t\tc'16\n\t}\n\tc'8\n}"


def test_RhythmTreeNode___call___03():

    rtm = '(1 (1 (2 (1 (2 (1 1)) 1)) 2))'
    tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
    result = tree((1, 4))

    assert result[0].lilypond_format == "\\times 4/5 {\n\tc'16\n\tc'32\n\tc'32\n\tc'32\n\tc'32\n\tc'8\n}"




import math


def partition_sequence_extended_to_counts(sequence, counts, overhang=True):
    '''.. versionadded:: 2.0

    Partition sequence extended to counts.

    Example 1. Partition sequence extended to counts with overhang::

        >>> sequencetools.partition_sequence_extended_to_counts(
        ...     [1, 2, 3, 4], [6, 6, 6], overhang=True)
        [[1, 2, 3, 4, 1, 2], [3, 4, 1, 2, 3, 4], [1, 2, 3, 4, 1, 2], [3, 4]]

    Example 2. Partition sequence extended to coutns without overhang::

        >>> sequencetools.partition_sequence_extended_to_counts(
        ...     [1, 2, 3, 4], [6, 6, 6], overhang=False)
        [[1, 2, 3, 4, 1, 2], [3, 4, 1, 2, 3, 4], [1, 2, 3, 4, 1, 2]]

    Return sequence of sequence objects.
    '''
    from abjad.tools import sequencetools

    n = int(math.ceil(float(sum(counts)) / len(sequence)))

    sequence = sequencetools.repeat_sequence_n_times(sequence, n)

    return sequencetools.partition_sequence_by_counts(sequence, counts, cyclic=False, overhang=overhang)



from abjad.tools.selectiontools.Selection import Selection


class Lineage(Selection):
    r'''Abjad model of Component lineage:

    ::

        >>> score = Score()
        >>> score.append(Staff(r"""\new Voice = "Treble Voice" { c'4 }""",
        ... name='Treble Staff'))
        >>> score.append(Staff(r"""\new Voice = "Bass Voice" { b,4 }""",
        ... name='Bass Staff'))
    
    ::

        >>> f(score)
        \new Score <<
            \context Staff = "Treble Staff" {
                \context Voice = "Treble Voice" {
                    c'4
                }
            }
            \context Staff = "Bass Staff" {
                \context Voice = "Bass Voice" {
                    b,4
                }
            }
        >>

    ::

        >>> for x in componenttools.Lineage(score): x
        ...
        Score<<2>>
        Staff-"Treble Staff"{1}
        Voice-"Treble Voice"{1}
        Note("c'4")
        Staff-"Bass Staff"{1}
        Voice-"Bass Voice"{1}
        Note('b,4')

    ::

        >>> for x in componenttools.Lineage(score['Bass Voice']): x
        ...
        Score<<2>>
        Staff-"Bass Staff"{1}
        Voice-"Bass Voice"{1}
        Note('b,4')

    Return Lineage instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_component',)

    ### INITIALIZER ###

    def __init__(self, component):
        from abjad.tools import componenttools

        assert isinstance(component, componenttools.Component)
        music = componenttools.get_lineage_of_component(component)
        Selection.__init__(self, music)
        self._component = component

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def component(self):
        '''The component from which the selection was derived.'''
        return self._component




from abjad.tools import componenttools


def remove_trivial_tuplets_in_expr(expr):
    r'''Remove trivial tuplets in `expr`::

        >>> tuplet_1 = tuplettools.FixedDurationTuplet(Duration(1, 4), "c'8 d'8 e'8")
        >>> tuplet_2 = tuplettools.FixedDurationTuplet(Duration(1, 4), "c'8 d'8")
        >>> staff = Staff([tuplet_1, tuplet_2])

    ::

        >>> f(staff)
        \new Staff {
            \times 2/3 {
                c'8
                d'8
                e'8
            }
            {
                c'8
                d'8
            }
        }

    ::

        >>> tuplettools.remove_trivial_tuplets_in_expr(staff)

    ::

        >>> f(staff)
        \new Staff {
            \times 2/3 {
                c'8
                d'8
                e'8
            }
            c'8
            d'8
        }

    Replace trivial tuplets with plain leaves.

    Return none.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import tuplettools

    for tuplet in list(iterationtools.iterate_components_in_expr(expr, tuplettools.Tuplet)):
        if tuplet.is_trivial:
            componenttools.move_parentage_and_spanners_from_components_to_components(
                [tuplet], tuplet[:])



import copy
from abjad.tools import measuretools
from abjad.tools import sequencetools
from abjad.tools import voicetools
from abjad.tools.quantizationtools.QTarget import QTarget


class MeasurewiseQTarget(QTarget):
    '''A measure-wise quantization target.

    Not composer-safe.

    Used internally by ``Quantizer``.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def beats(self):
        return tuple([beat for item in self.items for beat in item.beats])

    @property
    def item_klass(self):
        from abjad.tools import quantizationtools
        return quantizationtools.QTargetMeasure

    ### PRIVATE METHODS ###

    def _notate(self, grace_handler, attack_point_optimizer, attach_tempo_marks):
        voice = voicetools.Voice()

        # generate the first
        q_target_measure = self.items[0]
        measure = measuretools.Measure(q_target_measure.time_signature)
        for beat in q_target_measure.beats:
            measure.extend(beat.q_grid(beat.beatspan))
        if attach_tempo_marks:
            copy.copy(q_target_measure.tempo)(measure)
        voice.append(measure)

        # generate the rest pairwise, comparing tempi
        for q_target_measure_one, q_target_measure_two in \
            sequencetools.iterate_sequence_pairwise_strict(self.items):
            measure = measuretools.Measure(q_target_measure_two.time_signature)
            for beat in q_target_measure_two.beats:
                measure.extend(beat.q_grid(beat.beatspan))
            if (q_target_measure_two.tempo != q_target_measure_one.tempo) and attach_tempo_marks:
                copy.copy(q_target_measure_two.tempo)(measure)
            voice.append(measure)

        # apply tie chains, pitches, grace containers
        self._notate_leaves_pairwise(voice, grace_handler)

        # partition tiechains in each measure
        for measure in voice:
            attack_point_optimizer(measure)

        return voice




def make_repeated_notes_from_time_signature(time_signature, pitch="c'"):
    '''.. versionadded:: 2.0

    Make repeated notes from `time_signature`::

        >>> notetools.make_repeated_notes_from_time_signature((5, 32))
        [Note("c'32"), Note("c'32"), Note("c'32"), Note("c'32"), Note("c'32")]

    Make repeated notes with `pitch` from `time_signature`::

        >>> notetools.make_repeated_notes_from_time_signature((5, 32), pitch="d''")
        [Note("d''32"), Note("d''32"), Note("d''32"), Note("d''32"), Note("d''32")]

    Return list of notes.
    '''
    from abjad.tools import contexttools
    from abjad.tools import notetools

    # afford basic input polymorphism
    time_signature = contexttools.TimeSignatureMark(time_signature)

    # check input
    if time_signature.has_non_power_of_two_denominator:
        raise NotImplementedError(
            'TODO: extend this function for time signatures with a non-power-of-two denominators.')

    # make and return repeated notes
    return time_signature.numerator * notetools.Note(pitch, (1, time_signature.denominator))



from abjad.tools.abctools.AbjadObject import AbjadObject


class Selection(AbjadObject):
    '''.. versionadded:: 2.9

    Selection taken from a single score.

    Selection objects will eventually pervade the system and model all user selections.

    This means that selection objects will eventually serve as input
    to most functions in the API. Selection objects will also
    eventually be returned as output from most functions in the API.

    Selections are immutable and never change after instantiation.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_music',)

    _default_positional_input_arguments = ([], )

    ### INITIALIZER ###

    def __init__(self, music):
        if music is None:
            music = ()
        elif isinstance(music, (tuple, list, type(self))):
            music = tuple(music)
        else:
            music = (music, )
        self._music = tuple(music)

    ### SPECIAL METHODS ###

    def __add__(self, expr):
        assert isinstance(expr, (type(self), list, tuple))
        if isinstance(expr, type(self)):
            music = self.music + expr.music
            return type(self)(music)
        # eventually remove this permissive branch and force the use of selections only
        elif isinstance(expr, (tuple, list)):
            music = self.music + tuple(expr)
        return type(self)(music)
        
    def __contains__(self, expr):
        return expr in self.music

    def __eq__(self, expr):
        if isinstance(expr, type(self)):
            return self.music == expr.music
        # eventually remove this second, more permissive branch altogether
        elif isinstance(expr, (list, tuple)):
            return self.music == tuple(expr)

    def __getitem__(self, expr):
        return self.music.__getitem__(expr)

    def __len__(self):
        return len(self.music)

    def __ne__(self, expr):
        return not self == expr

    def __radd__(self, expr):
        assert isinstance(expr, (type(self), list, tuple))
        if isinstance(expr, type(self)):
            music = expr.music + self.music
            return type(self)(music)
        # eventually remove this permissive branch and force the use of selections only
        elif isinstance(expr, (tuple, list)):
            music = tuple(expr) + self.music
        return type(self)(music)

    def __repr__(self):
        return '{}{!r}'.format(self._class_name, self.music)
    
    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def music(self):
        '''Read-only tuple of components in selection.
        '''
        return self._music

    @property
    def start_offset(self):
        '''Read-only start offset of earliest component in selection.
        '''
        return min(x.start_offset for x in self)

    @property
    def stop_offset(self):
        '''Read-only stop offset of earliest component in selection.
        '''
        return max(x.stop_offset for x in self)

    @property
    def timespan(self):
        '''Read-only timespan of selection.
        '''
        from abjad.tools import timespantools
        return timespantools.Timespan(self.start_offset, self.stop_offset)



from abjad.tools import componenttools
from abjad.tools import iterationtools
from abjad.tools import measuretools
from abjad.tools.wellformednesstools.Check import Check


class NestedMeasureCheck(Check):
    '''Do we have any nested measures?'''

    def _run(self, expr):
        violators = []
        total = 0
        for t in iterationtools.iterate_measures_in_expr(expr):
            if componenttools.get_first_instance_of_klass_in_proper_parentage_of_component(t, measuretools.Measure):
                violators.append(t)
            total += 1
        return violators, total



from abjad import *


def test_tuplettools_leaf_to_tuplet_with_n_notes_of_equal_written_duration_01():
    '''Divide a leaf of 3/16 into 1, ..., 5 parts.'''

    t = tuplettools.leaf_to_tuplet_with_n_notes_of_equal_written_duration(Note("c'8."), 1, is_diminution=False)
    assert t.lilypond_format == "{\n\tc'8.\n}"

    t = tuplettools.leaf_to_tuplet_with_n_notes_of_equal_written_duration(Note("c'8."), 2, is_diminution=False)
    assert t.lilypond_format == "{\n\tc'16.\n\tc'16.\n}"

    t = tuplettools.leaf_to_tuplet_with_n_notes_of_equal_written_duration(Note("c'8."), 3, is_diminution=False)
    assert t.lilypond_format == "{\n\tc'16\n\tc'16\n\tc'16\n}"

    t = tuplettools.leaf_to_tuplet_with_n_notes_of_equal_written_duration(Note("c'8."), 4, is_diminution=False)
    assert t.lilypond_format == "{\n\tc'32.\n\tc'32.\n\tc'32.\n\tc'32.\n}"

    t = tuplettools.leaf_to_tuplet_with_n_notes_of_equal_written_duration(Note("c'8."), 5, is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 8/5 {\n\tc'64.\n\tc'64.\n\tc'64.\n\tc'64.\n\tc'64.\n}"


def test_tuplettools_leaf_to_tuplet_with_n_notes_of_equal_written_duration_02():
    '''Divide a leaf of 3/16 into 1, ..., 5 parts.'''

    t = tuplettools.leaf_to_tuplet_with_n_notes_of_equal_written_duration(Note("c'8."), 1, is_diminution=True)
    assert t.lilypond_format == "{\n\tc'8.\n}"

    t = tuplettools.leaf_to_tuplet_with_n_notes_of_equal_written_duration(Note("c'8."), 2, is_diminution=True)
    assert t.lilypond_format == "{\n\tc'16.\n\tc'16.\n}"

    t = tuplettools.leaf_to_tuplet_with_n_notes_of_equal_written_duration(Note("c'8."), 3, is_diminution=True)
    assert t.lilypond_format == "{\n\tc'16\n\tc'16\n\tc'16\n}"

    t = tuplettools.leaf_to_tuplet_with_n_notes_of_equal_written_duration(Note("c'8."), 4, is_diminution=True)
    assert t.lilypond_format == "{\n\tc'32.\n\tc'32.\n\tc'32.\n\tc'32.\n}"

    # TODO: DECIDE ON DOTTED VALUES #
    #t = tuplettools.leaf_to_tuplet_with_n_notes_of_equal_written_duration(Note("c'8."), 5, is_diminution=True)
    #assert t.lilypond_format == "\\fraction \\times 3/5 {\n\tc'16\n\tc'16\n\tc'16\n\tc'16\n\tc'16\n}"



from abjad.tools import durationtools
from abjad.tools import leaftools


def make_percussion_note(pitch, total_duration, max_note_duration=(1, 8)):
    '''.. versionadded:: 1.0

    Make percussion note::

        >>> notetools.make_percussion_note(2, (1, 4), (1, 8))
        [Note("d'8"), Rest('r8')]

    ::

        >>> notetools.make_percussion_note(2, (1, 64), (1, 8))
        [Note("d'64")]

    ::

        >>> notetools.make_percussion_note(2, (5, 64), (1, 8))
        [Note("d'16"), Rest('r64')]

    ::

        >>> notetools.make_percussion_note(2, (5, 4), (1, 8))
        [Note("d'8"), Rest('r1'), Rest('r8')]

    Return list of newly constructed note followed by zero or more newly constructed rests.

    Durations of note and rests returned will sum to `total_duration`.

    Duration of note returned will be no greater than `max_note_duration`.

    Duration of rests returned will sum to note duration taken from `total_duration`.

    Useful for percussion music where attack duration is negligible and tied notes undesirable.
    '''
    from abjad.tools import notetools
    from abjad.tools import resttools

    # check input
    total_duration = durationtools.Duration(total_duration)
    max_note_duration = durationtools.Duration(max_note_duration)

    # make note and rest
    if max_note_duration < total_duration:
        rest_duration = total_duration - max_note_duration
        r = resttools.make_tied_rest(rest_duration)
        n = notetools.make_tied_note(pitch, max_note_duration)
    else:
        n = leaftools.make_tied_leaf(notetools.Note, total_duration, pitches=pitch, tie_parts=False)
        if 1 < len(n):
            for i in range(1, len(n)):
                n[i] = resttools.Rest(n[i])
        r = []

    # return list of percussion note followed by rest
    return n + r



from abjad.tools import *
import os


def write_test_output(score, full_file_name, test_function_name,
    cache_ly=False, cache_pdf=False, go=False, render_pdf=False):
    r'''Write test output.
    '''
    from experimental.tools import testtools
    if go: cache_ly = cache_pdf = render_pdf = True
    if not any([cache_ly, cache_pdf, render_pdf]):
        return
    lilypond_file = lilypondfiletools.make_floating_time_signature_lilypond_file(score)
    testtools.configure_multiple_voice_rhythmic_staves(lilypond_file)
    title_lines = test_function_name_to_title_lines(test_function_name)
    lilypond_file.header_block.title = markuptools.make_centered_title_markup(
        title_lines, font_size=6, vspace_before=2, vspace_after=4)
    lilypond_file.score.set.proportionalNotationDuration = schemetools.SchemeMoment((1, 48))
    parent_directory_name = os.path.dirname(full_file_name)
    if render_pdf:
        iotools.show(lilypond_file)
    if cache_pdf:
        file_name = '{}.pdf'.format(test_function_name)
        pdf_path_name = os.path.join(parent_directory_name, file_name)
        iotools.write_expr_to_pdf(lilypond_file, pdf_path_name)
    if cache_ly:
        file_name = '{}.ly'.format(test_function_name)
        ly_path_name = os.path.join(parent_directory_name, file_name)
        file(ly_path_name, 'w').write(score.lilypond_format)


def test_function_name_to_title_lines(test_function_name):
    title_lines = []
    test_function_name = test_function_name[5:]
    if '__' in test_function_name:
        left_half, right_half = test_function_name.split('__')
        left_half = left_half.replace('_', ' ')
        title_lines.append(left_half)
        parts = right_half.split('_')
    else:
        parts = test_function_name.split('_')
    test_number = int(parts[-1])
    parts.pop(-1)
    if parts[0][0].isupper() and 1 < len(parts[0]):
        title_lines.append(parts.pop(0))
    lengths = [len(part) for part in parts]
    if 35 < sum(lengths):
        halves = sequencetools.partition_sequence_by_ratio_of_weights(lengths, [1, 1])
        left_count = len(halves[0])
        right_count = len(halves[-1])
        assert left_count + right_count == len(lengths)
        left_parts = parts[:left_count]
        title_lines.append(' '.join(left_parts))
        right_parts = parts[-right_count:]
        right_parts.append(str(test_number))
        title_lines.append(' '.join(right_parts))
    else:
        title_words = ' '.join(parts)
        if 'schematic example' in title_words:
            space = ''
        else:
            space = ' '
        title = '{}{}{}'.format(title_words, space, test_number)
        title_lines.append(title)
    return title_lines



import abc
import copy
import inspect
import pprint
from abjad.tools import datastructuretools
from abjad.tools import sequencetools
from abjad.tools.abctools import AbjadObject


class SearchTree(AbjadObject):
    '''Abstract base class from which concrete ``SearchTree`` subclasses inherit.

    ``SearchTrees`` encapsulate strategies for generating collections of ``QGrids``,
    given a set of ``QEventProxy`` instances as input.

    They allow composers to define the degree and quality of nested rhythmic
    subdivisions in the quantization output.  That is to say, they allow composers
    to specify what sorts of tuplets and ratios of pulses may be contained within
    other tuplets, to arbitrary levels of nesting.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_definition',)

    ### INITIALIZER ###

    def __init__(self, definition=None):
        if definition is None:
            definition = self.default_definition
        else:
            assert self._is_valid_definition(definition)
        self._definition = definition

    ### SPECIAL METHODS ###

    def __call__(self, q_grid):
        from abjad.tools import quantizationtools

        assert isinstance(q_grid, quantizationtools.QGrid)
        new_q_grids = []
        commands = self._generate_all_subdivision_commands(q_grid)
        for command in commands:
            new_q_grid = copy.copy(q_grid)
            q_events = new_q_grid.subdivide_leaves(command)
            new_q_grid.fit_q_events(q_events)
            new_q_grids.append(new_q_grid)
        return new_q_grids

    def __eq__(self, other):
        if type(self) == type(other):
            if self.definition == other.definition:
                return True
        return False

    def __getnewargs__(self):
        return (self.definition,)

    def __getstate__(self):
        state = {}
        for klass in inspect.getmro(self.__class__):  
            if hasattr(klass, '__slots__'):
                for slot in klass.__slots__:
                    if slot not in state:
                        state[slot] = getattr(self, slot)
        return state
    
    def __repr__(self):
        result = ['{}('.format(self._class_name)]
        definition = pprint.pformat(self.definition, indent=4, width=64).splitlines()
        result.append('\tdefinition={}'.format(definition[0]))
        result.extend(['\t' + x for x in definition[1:]])
        result.append('\t)')
        return '\n'.join(result)

    def __setstate__(self, state):
        for key, value in state.iteritems():
            setattr(self, key, value)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @abc.abstractproperty
    def default_definition(self):
        '''The default search tree definition.

        Return dictionary.
        '''
        raise NotImplemented

    @property
    def definition(self):
        '''The search tree definition.

        Return dictionary.
        '''
        return copy.deepcopy(self._definition)

    ### PRIVATE METHODS ###

    def _find_divisible_leaf_indices_and_subdivisions(self, q_grid):
        # TODO: This should actually check for all QEvents which fall within the leaf's duration,
        # including QEvents attached to the next leaf
        # It may be prudent to actually store QEvents in two lists: before_offset and after_offset
        indices, subdivisions = [], []

        leaves = q_grid.leaves
        i = 0
        for leaf_one, leaf_two in sequencetools.iterate_sequence_pairwise_strict(leaves):
            if (leaf_one.succeeding_q_event_proxies or leaf_two.preceding_q_event_proxies) \
                and leaf_one.is_divisible:
                if len(leaf_one.q_event_proxies) == 1 and leaf_one.q_event_proxies[0].offset == leaf_one.start_offset:
                    pass # a perfect match, don't bother to continue subdivision
                else:
                    parentage_ratios = leaf_one.parentage_ratios
                    leaf_subdivisions = self._find_leaf_subdivisions(parentage_ratios)
                    if leaf_subdivisions:
                        indices.append(i)
                        subdivisions.append(tuple(leaf_subdivisions))
            i += 1
        return indices, subdivisions

    @abc.abstractmethod
    def _find_leaf_subdivisions(self, leaf):
        raise NotImplemented

    def _generate_all_subdivision_commands(self, q_grid):
        indices, subdivisions = self._find_divisible_leaf_indices_and_subdivisions(q_grid)
        if not indices:
            return ()
        combinations = [tuple(x) for x in sequencetools.yield_outer_product_of_sequences(subdivisions)]
        return tuple(tuple(zip(indices, combo)) for combo in combinations)

    def _get_tools_package_qualified_repr_pieces(self, is_indented=True):
        indent = '\t'
        if not is_indented:
            indent = ''
        result = ['{}('.format(self._tools_package_qualified_class_name)]
        definition = pprint.pformat(self.definition, indent=4, width=64).splitlines()
        result.append('{}definition={}'.format(indent, definition[0]))
        result.extend([indent + x for x in definition[1:]])
        result.append('{})'.format(indent))
        return result

    @abc.abstractmethod
    def _is_valid_definition(self, definition):
        raise NotImplemented



from abjad.tools import contexttools


def apply_dynamic_marks(score):

    voice = score['Bell Voice']
    contexttools.DynamicMark('ppp')(voice[0][1])
    contexttools.DynamicMark('pp')(voice[8][1])
    contexttools.DynamicMark('p')(voice[18][1])
    contexttools.DynamicMark('mp')(voice[26][1])
    contexttools.DynamicMark('mf')(voice[34][1])
    contexttools.DynamicMark('f')(voice[42][1])
    contexttools.DynamicMark('ff')(voice[52][1])
    contexttools.DynamicMark('fff')(voice[60][1])
    contexttools.DynamicMark('ff')(voice[68][1])
    contexttools.DynamicMark('f')(voice[76][1])
    contexttools.DynamicMark('mf')(voice[84][1])
    contexttools.DynamicMark('pp')(voice[-1][0])

    voice = score['First Violin Voice']
    contexttools.DynamicMark('ppp')(voice[6][1])
    contexttools.DynamicMark('pp')(voice[15][0])
    contexttools.DynamicMark('p')(voice[22][3])
    contexttools.DynamicMark('mp')(voice[31][0])
    contexttools.DynamicMark('mf')(voice[38][3])
    contexttools.DynamicMark('f')(voice[47][0])
    contexttools.DynamicMark('ff')(voice[55][2])
    contexttools.DynamicMark('fff')(voice[62][2])

    voice = score['Second Violin Voice']
    contexttools.DynamicMark('pp')(voice[7][0])
    contexttools.DynamicMark('p')(voice[12][0])
    contexttools.DynamicMark('p')(voice[16][0])
    contexttools.DynamicMark('mp')(voice[25][1])
    contexttools.DynamicMark('mf')(voice[34][1])
    contexttools.DynamicMark('f')(voice[44][1])
    contexttools.DynamicMark('ff')(voice[54][0])
    contexttools.DynamicMark('fff')(voice[62][1])

    voice = score['Viola Voice']
    contexttools.DynamicMark('p')(voice[8][0])
    contexttools.DynamicMark('mp')(voice[19][1])
    contexttools.DynamicMark('mf')(voice[30][0])
    contexttools.DynamicMark('f')(voice[36][0])
    contexttools.DynamicMark('f')(voice[42][0])
    contexttools.DynamicMark('ff')(voice[52][0])
    contexttools.DynamicMark('fff')(voice[62][0])

    voice = score['Cello Voice']
    contexttools.DynamicMark('p')(voice[10][0])
    contexttools.DynamicMark('mp')(voice[21][0])
    contexttools.DynamicMark('mf')(voice[31][0])
    contexttools.DynamicMark('f')(voice[43][0])
    contexttools.DynamicMark('ff')(voice[52][1])
    contexttools.DynamicMark('fff')(voice[62][0])

    voice = score['Bass Voice']
    contexttools.DynamicMark('mp')(voice[14][0])
    contexttools.DynamicMark('mf')(voice[27][0])
    contexttools.DynamicMark('f')(voice[39][0])
    contexttools.DynamicMark('ff')(voice[51][0])
    contexttools.DynamicMark('fff')(voice[62][0])




from abjad import *


def test_RhythmTreeContainer_rtm_format_01(): 

    leaf_a = rhythmtreetools.RhythmTreeLeaf(duration=3)
    leaf_b = rhythmtreetools.RhythmTreeLeaf(duration=3)
    leaf_c = rhythmtreetools.RhythmTreeLeaf(duration=2)
    subcontainer = rhythmtreetools.RhythmTreeContainer(
        duration=2, children=[leaf_b, leaf_c])
    leaf_d = rhythmtreetools.RhythmTreeLeaf(duration=1)
    container = rhythmtreetools.RhythmTreeContainer(
        duration=1, children=[leaf_a, subcontainer, leaf_d])
    
    assert subcontainer.rtm_format == '(2 (3 2))'
    assert container.rtm_format == '(1 (3 (2 (3 2)) 1))'



from abjad.tools import leaftools


def add_markup_to_end_of_score(score, markup, extra_offset=None):
    r'''.. versionadded:: 2.0

    Add `markup` to end of `score`::

        >>> staff = Staff("c'4 d'4 e'4 f'4")
        >>> markup = r'\italic \right-column { "Bremen - Boston - LA." "Jul 2010 - May 2011." }'
        >>> markup = markuptools.Markup(markup, Down)
        >>> markup = scoretools.add_markup_to_end_of_score(staff, markup, (4, -2))

    ::

        >>> z(markup)
        markuptools.Markup((
            markuptools.MarkupCommand(
                'italic',
                markuptools.MarkupCommand(
                    'right-column',
                    [
                        'Bremen - Boston - LA.',
                        'Jul 2010 - May 2011.'
                    ]
                    )
                ),
            ),
            direction=Down
            )

    ::

        >>> f(staff)
        \new Staff {
            c'4
            d'4
            e'4
            \once \override TextScript #'extra-offset = #'(4 . -2)
            f'4 _ \markup { 
                \italic
                    \right-column
                        {
                            "Bremen - Boston - LA."
                            "Jul 2010 - May 2011."
                        }
                }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return `markup`.
    '''
    from abjad.tools import markuptools

    last_leaf = leaftools.get_nth_leaf_in_expr(score, -1)
    # TODO: copy markup direction from markup input
    markup = markuptools.Markup(markup, Down)(last_leaf)

    if extra_offset is not None:
        last_leaf.override.text_script.extra_offset = extra_offset

    return markup



import itertools


def report_time_signature_distribution(expr):
    r'''.. versionadded:: 2.0

    Report time signature distribution of `expr`::

        >>> staff = Staff(r"abj: | 2/4 c'4 d'4 || 2/4 e'4 f'4 || 2/4 g'2 |" \
        ...     "| 5/8 c'8 d'8 e'8 f'8 g'8 |")

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/4
                c'4
                d'4
            }
            {
                e'4
                f'4
            }
            {
                g'2
            }
            {
                \time 5/8
                c'8
                d'8
                e'8
                f'8
                g'8
            }
        }

    ::

        >>> print measuretools.report_time_signature_distribution(staff)
            2/4 3
            5/8 1

    Return string.
    '''
    from abjad.tools import contexttools
    from abjad.tools import iterationtools

    time_signatures = []
    for measure in iterationtools.iterate_measures_in_expr(expr):
        time_signatures.append(contexttools.get_effective_time_signature(measure))

    time_signatures.sort()

    result = ''
    for key, values_generator in itertools.groupby(time_signatures):
        result += '\t%s\t%s\n' % (key, len(list(values_generator)))

    return result



from abjad import *


def test_Timespan_stops_at_or_after_offset_01():
    timespan = timespantools.Timespan(0, 10)
    offset = durationtools.Offset(-5)
    assert timespan.stops_at_or_after_offset(offset)

def test_Timespan_stops_at_or_after_offset_02():
    timespan = timespantools.Timespan(0, 10)
    offset = durationtools.Offset(0)
    assert timespan.stops_at_or_after_offset(offset)

def test_Timespan_stops_at_or_after_offset_03():
    timespan = timespantools.Timespan(0, 10)
    offset = durationtools.Offset(5)
    assert timespan.stops_at_or_after_offset(offset)

def test_Timespan_stops_at_or_after_offset_04():
    timespan = timespantools.Timespan(0, 10)
    offset = durationtools.Offset(10)
    assert timespan.stops_at_or_after_offset(offset)

def test_Timespan_stops_at_or_after_offset_05():
    timespan = timespantools.Timespan(0, 10)
    offset = durationtools.Offset(15)
    assert not timespan.stops_at_or_after_offset(offset)





from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import sequencetools


def make_gridded_test_rhythm(grid_length, rhythm_number, denominator=16):
    r'''.. versionadded:: 2.11

    Make test rhythm number `rhythm_number` that fits `grid_length`.
    
    Return selection of one or more possibly tied notes.

    Example 1. The eight test rhythms that fit a length-``4`` grid::

        >>> for rhythm_number in range(8):
        ...     notes = timesignaturetools.make_gridded_test_rhythm(
        ...         4, rhythm_number, denominator=4)
        ...     measure = Measure((4, 4), notes)
        ...     print '{}\t{}'.format(rhythm_number, measure)
        ... 
        0   |4/4, c'1|
        1   |4/4, c'2., c'4|
        2   |4/4, c'2, c'4, c'4|
        3   |4/4, c'2, c'2|
        4   |4/4, c'4, c'4, c'2|
        5   |4/4, c'4, c'4, c'4, c'4|
        6   |4/4, c'4, c'2, c'4|
        7   |4/4, c'4, c'2.|

    Example 2. The sixteenth test rhythms for that a length-``5`` grid::

        >>> for rhythm_number in range(16):
        ...     notes = timesignaturetools.make_gridded_test_rhythm(
        ...         5, rhythm_number, denominator=4)
        ...     measure = Measure((5, 4), notes)
        ...     print '{}\t{}'.format(rhythm_number, measure)
        ... 
        0   |5/4, c'1, c'4|
        1   |5/4, c'1, c'4|
        2   |5/4, c'2., c'4, c'4|
        3   |5/4, c'2., c'2|
        4   |5/4, c'2, c'4, c'2|
        5   |5/4, c'2, c'4, c'4, c'4|
        6   |5/4, c'2, c'2, c'4|
        7   |5/4, c'2, c'2.|
        8   |5/4, c'4, c'4, c'2.|
        9   |5/4, c'4, c'4, c'2, c'4|
        10  |5/4, c'4, c'4, c'4, c'4, c'4|
        11  |5/4, c'4, c'4, c'4, c'2|
        12  |5/4, c'4, c'2, c'2|
        13  |5/4, c'4, c'2, c'4, c'4|
        14  |5/4, c'4, c'2., c'4|
        15  |5/4, c'4, c'1|

    Use for testing metrical hierarchy establishment.
    '''
    from abjad.tools import notetools

    # check input
    assert mathtools.is_positive_integer(grid_length)
    assert isinstance(rhythm_number, int)
    assert mathtools.is_positive_integer_power_of_two(denominator)

    # find count of all rhythms that fit grid length
    rhythm_count = 2 ** (grid_length - 1)

    # read rhythm number cyclically to allow large and negative rhythm numbers
    rhythm_number = rhythm_number % rhythm_count

    # find binary representation of rhythm
    binary_representation = mathtools.integer_to_binary_string(rhythm_number)
    binary_representation = binary_representation.zfill(grid_length) 

    # partition binary representation of rhythm
    parts = sequencetools.partition_sequence_by_value_of_elements(binary_representation)
    
    # find durations
    durations = [durationtools.Duration(len(part), denominator) for part in parts]

    # make notes
    notes = notetools.make_notes([0], durations)

    # return notes
    return notes



from abjad import *


def test_BookpartBlock_01():

    bookpart_block = lilypondfiletools.BookpartBlock()

    r'''
    \bookpart {}
    '''

    assert bookpart_block.lilypond_format == '\\bookpart {}'



import os
import subprocess


def graph(expr, image_format='pdf', layout='dot'):
    '''Graph `expr` with graphviz, and open resulting image in the default image viewer:

    ::

        >>> rtm_syntax = '(3 ((2 (2 1)) 2))'
        >>> rhythm_tree = rhythmtreetools.RhythmTreeParser()(rtm_syntax)[0]
        >>> print rhythm_tree.pretty_rtm_format
        (3 (
            (2 (
                2
                1))
            2))

    ::

        >>> graph(rhythm_tree) # doctest: +SKIP

    Return None.
    '''

    from abjad import ABJCFG
    from abjad.tools import iotools
    from abjad.tools.iotools._open_file import _open_file
    from abjad.tools.iotools._verify_output_directory import _verify_output_directory

    assert image_format in ('pdf', 'png')
    assert layout in ('circo', 'dot', 'fdp', 'neato', 'osage', 'sfdp', 'twopi')

    if isinstance(expr, str):
        graphviz_format = expr
    else:
        graphviz_format = expr.graphviz_format

    current_directory = os.path.abspath('.')
    ABJADOUTPUT = ABJCFG['abjad_output']
    _verify_output_directory(ABJADOUTPUT)
    dot_path = os.path.join(ABJADOUTPUT, iotools.get_next_output_file_name(file_extension='dot'))
    img_path = os.path.join(ABJADOUTPUT, dot_path.replace('dot', 'pdf'))

    with open(dot_path, 'w') as f:
        f.write(graphviz_format)

    command = '{} -v -T{} {} -o {}'.format(layout, image_format, dot_path, img_path)
    subprocess.call(command, shell=True)

    pdf_viewer = ABJCFG['pdf_viewer']
    ABJADOUTPUT = ABJCFG['abjad_output']
    _open_file(img_path, pdf_viewer)





def p(arg, language='english'):
    r'''.. versionadded:: 2.7

    Parse `arg` as LilyPond string::

        >>> p("{c'4 d'4 e'4 f'4}")
        {c'4, d'4, e'4, f'4}

    ::

        >>> container = _
        >>> f(container)
        {
            c'4
            d'4
            e'4
            f'4
        }

    A pitch-name language may also be specified.

    ::

        >>> p("{c'8 des' e' fis'}", language='nederlands')
        {c'8, df'8, e'8, fs'8}

    Return Abjad expression.
    '''
    # TODO: lilypondparsertools should NOT depend on iotools.
    #       The direction of dependency should be the other way around.
    from abjad.tools import rhythmtreetools
    from abjad.tools import lilypondparsertools

    if arg.startswith('abj:'):
        return lilypondparsertools.parse_reduced_ly_syntax(arg[4:])
    elif arg.startswith('rtm:'):
        return rhythmtreetools.parse_rtm_syntax(arg[4:])
    return lilypondparsertools.LilyPondParser(default_language=language)(arg)



# -*- encoding: utf-8 -*-

from abjad import *
from abjad.demos.desordre.make_desordre_staff import make_desordre_staff


def make_desordre_score(pitches):
    '''Returns a complete PianoStaff with Ligeti music!'''

    assert len(pitches) == 2
    piano_staff = scoretools.PianoStaff()

    # build the music...
    for hand in pitches:
        staff = make_desordre_staff(hand)
        piano_staff.append(staff)

    # set clef and key signature to left hand staff...
    contexttools.ClefMark('bass')(piano_staff[1])
    contexttools.KeySignatureMark('b', 'major')(piano_staff[1])

    # wrap the piano staff in a score, and return
    score = Score([piano_staff])

    return score



from abjad import *


def test_RhythmTreeContainer___contains___01():

    leaf_a = rhythmtreetools.RhythmTreeLeaf(duration=1)
    leaf_b = rhythmtreetools.RhythmTreeLeaf(duration=1)
    leaf_c = rhythmtreetools.RhythmTreeLeaf(duration=1)

    subcontainer = rhythmtreetools.RhythmTreeContainer(duration=1, children=[leaf_b])

    container = rhythmtreetools.RhythmTreeContainer(duration=1, children=[leaf_a, subcontainer])

    assert leaf_a in container
    assert leaf_b not in container
    assert leaf_c not in container



from abjad.tools.durationtools import Offset
from abjad.tools import timeintervaltools


def test_TimeInterval_storage_format_01():

    t = timeintervaltools.TimeInterval(0, 1, {
        'a': timeintervaltools.TimeInterval(3, 4, {'x': 'y'}), 'b': 3
    })

    r'''
    timeintervaltools.TimeInterval(
        Offset(0, 1),
        Offset(1, 1),
        {
            'a': timeintervaltools.TimeInterval(
                Offset(3, 1),
                Offset(4, 1),
                {
                    'x': 'y',
                }),
            'b': 3,
        })
    '''

    assert t.storage_format == "timeintervaltools.TimeInterval(\n\tOffset(0, 1),\n\tOffset(1, 1),\n\t{\n\t\t'a': timeintervaltools.TimeInterval(\n\t\t\tOffset(3, 1),\n\t\t\tOffset(4, 1),\n\t\t\t{\n\t\t\t\t'x': 'y',\n\t\t\t}),\n\t\t'b': 3,\n\t})"



from abjad.tools.spannertools.Spanner import Spanner


class TextSpanner(Spanner):
    r'''.. versionadded:: 2.0

    Abjad text spanner::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> text_spanner = spannertools.TextSpanner(staff[:])

    ::

        >>> markup = markuptools.Markup(markuptools.MarkupCommand(
        ...     'bold', markuptools.MarkupCommand('italic', 'foo')))
        >>> text_spanner.override.text_spanner.bound_details__left__text = markup
        >>> markup = markuptools.Markup(
        ...     markuptools.MarkupCommand('draw-line', schemetools.SchemePair(0, -1)))
        >>> text_spanner.override.text_spanner.bound_details__right__text = markup
        >>> text_spanner.override.text_spanner.dash_fraction = 1

    ::

        >>> f(staff)
        \new Staff {
            \override TextSpanner #'bound-details #'left #'text = \markup { 
                \bold \italic foo }
            \override TextSpanner #'bound-details #'right #'text = \markup {
                \draw-line #'(0 . -1) }
            \override TextSpanner #'dash-fraction = #1
            c'8 \startTextSpan
            d'8
            e'8
            f'8 \stopTextSpan
            \revert TextSpanner #'bound-details #'left #'text
            \revert TextSpanner #'bound-details #'right #'text
            \revert TextSpanner #'dash-fraction
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Override LilyPond TextSpanner grob.

    Return text spanner.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None):
        Spanner.__init__(self, components)

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        pass

    def _format_right_of_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\startTextSpan')
        if self._is_my_last_leaf(leaf):
            result.append(r'\stopTextSpan')
        return result



def extend_in_parent_of_component(component, new_components, left=False, grow_spanners=True):
    r'''.. versionadded:: 2.10

    Extend `new_components` in parent of `component`.

    Example 1. Extend `new_component` in parent of `component`.
    Grow spanners:

    ::

        >>> voice = Voice("c'8 [ d'8 e'8 ]")

    ::

        >>> f(voice)
        \new Voice {
            c'8 [
            d'8
            e'8 ]
        }

    ::

        >>> new_components = [Note("c'8"), Note("d'8"), Note("e'8")]
        >>> componenttools.extend_in_parent_of_component(
        ...     voice.leaves[-1], new_components, grow_spanners=True)
        [Note("e'8"), Note("c'8"), Note("d'8"), Note("e'8")]

    ::

        >>> f(voice)
        \new Voice {
            c'8 [
            d'8
            e'8
            c'8
            d'8
            e'8 ]
        }

    Example 2. Extend `new_component` in parent of `component`.
    Do not grow spanners:

        >>> voice = Voice("c'8 [ d'8 e'8 ]")

    ::

        >>> f(voice)
        \new Voice {
            c'8 [
            d'8
            e'8 ]
        }

    ::
    
        >>> new_components = [Note("c'8"), Note("d'8"), Note("e'8")]
        >>> componenttools.extend_in_parent_of_component(
        ...     voice.leaves[-1], new_components, grow_spanners=False)
        [Note("e'8"), Note("c'8"), Note("d'8"), Note("e'8")]

    ::

        >>> f(voice)
        \new Voice {
            c'8 [
            d'8
            e'8 ]
            c'8
            d'8
            e'8
        }

    Example 3. Extend `new_components` left in parent of `component`.
    Grow spanners::

        >>> voice = Voice("c'8 [ d'8 e'8 ]")

    ::

        >>> f(voice)
        \new Voice {
            c'8 [
            d'8
            e'8 ]
        }

    ::

        >>> notes = [Note("c'8"), Note("d'8"), Note("e'8")]
        >>> componenttools.extend_in_parent_of_component(
        ...     voice[0], notes, left=True, grow_spanners=True)
        [Note("c'8"), Note("d'8"), Note("e'8"), Note("c'8")]

    ::

        >>> f(voice)
        \new Voice {
            c'8 [
            d'8
            e'8
            c'8
            d'8
            e'8 ]
        }

    Example 4. Extend `new_components` left in parent of `component`.
    Do not grow spanners::

        >>> voice = Voice("c'8 [ d'8 e'8 ]")

    ::

        >>> notes = [Note("c'8"), Note("d'8"), Note("e'8")]
        >>> componenttools.extend_in_parent_of_component(
        ...     voice[0], notes, left=True, grow_spanners=False)
        [Note("c'8"), Note("d'8"), Note("e'8"), Note("c'8")]

    ::

        >>> f(voice)
        \new Voice {
            c'8
            d'8
            e'8
            c'8 [
            d'8
            e'8 ]
        }

    Return `component` and `new_components` together in newly constructed list.
    '''
    from abjad.tools import componenttools
    from abjad.tools import spannertools

    # check input
    assert componenttools.all_are_components(new_components)

    # extend to the right of component
    if not left:

        # extend spanners if required
        if grow_spanners:
            insert_offset = component.stop_offset
            receipt = spannertools.get_spanners_that_dominate_components([component])
            for spanner, index in receipt:
                insert_component = spannertools.find_spanner_component_starting_at_exactly_score_offset(
                    spanner, insert_offset)
                if insert_component is not None:
                    insert_index = spanner.index(insert_component)
                else:
                    insert_index = len(spanner)
                for new_component in reversed(new_components):
                    spanner._insert(insert_index, new_component)
                    new_component._spanners.add(spanner)

        # find parent
        parent, start, stop = componenttools.get_parent_and_start_stop_indices_of_components([component])

        # extend new components in parent
        if parent is not None:
            if grow_spanners:
                for new_component in reversed(new_components):
                    new_component._switch(parent)
                    parent._music.insert(start + 1, new_component)
            else:
                after = stop + 1
                # to avoid slice assignment pychecker errors
                #parent[after:after] = new_components
                parent.__setitem__(slice(after, after), new_components)

        # return component and new components together
        return [component] + new_components

    # extend to the left of component
    else:

        if grow_spanners:
            offset = component.start_offset
            receipt = spannertools.get_spanners_that_dominate_components([component])
            for spanner, x in receipt:
                index = spannertools.find_index_of_spanner_component_at_score_offset(spanner, offset)
                for new_component in reversed(new_components):
                    spanner._insert(index, new_component)
                    new_component._spanners.add(spanner)

        parent, start, stop = componenttools.get_parent_and_start_stop_indices_of_components([component])

        if parent is not None:
            if grow_spanners:
                for new_component in reversed(new_components):
                    new_component._switch(parent)
                    parent._music.insert(start, new_component)
            else:
                # to avoid slice assignment pychecker errors
                #parent[start:start] = new_components
                parent.__setitem__(slice(start, start), new_components)

        return new_components + [component]



from abjad.tools import schemetools
from abjad.tools.abctools import AbjadObject


class MarkupCommand(AbjadObject):
    r'''Abjad model of a LilyPond markup command::

        >>> circle = markuptools.MarkupCommand('draw-circle', 2.5, 0.1, False)
        >>> square = markuptools.MarkupCommand('rounded-box', 'hello?')
        >>> line = markuptools.MarkupCommand('line', [square, 'wow!'])
        >>> rotate = markuptools.MarkupCommand('rotate', 60, line)
        >>> combine = markuptools.MarkupCommand('combine', rotate, circle)

    ::

        >>> print combine
        \combine \rotate #60 \line { \rounded-box hello? wow! } \draw-circle #2.5 #0.1 ##f

    Insert markup command in markup to attach to score components::

        >>> note = Note("c'4")

    ::

        >>> markup = markuptools.Markup(combine)

    ::

        >>> markup = markup(note)

    ::

        >>> f(note)
        c'4
            - \markup {
                \combine
                    \rotate
                        #60
                        \line
                            {
                                \rounded-box
                                    hello?
                                wow!
                            }
                    \draw-circle
                        #2.5
                        #0.1
                        ##f
                }

    ::

        >>> show(note) # doctest: +SKIP

    Markup commands are immutable.
    '''

    __slots__ = ('_args', '_command')

    def __init__(self, command, *args):
        assert isinstance(command, str) \
            and len(command) and command.find(' ') == -1
        self._command = command
        self._args = tuple(args)

    ### SPECIAL METHODS ###

    def __eq__(self, other):
        if isinstance(other, type(self)):
            if self.command == other.command:
                if self.args == other.args:
                    return True
        return False

    def __repr__(self):
        result = [repr(self.command)]
        result.extend([repr(x) for x in self.args])
        return '%s(%s)' % (type(self).__name__, ', '.join(result))

    def __str__(self):
        return self.lilypond_format

    ### PRIVATE PROPERTIES ###

    @property
    def _format_pieces(self):
        return self._get_format_pieces(is_indented=False)

    ### PUBLIC PROPERTIES ###

    @property
    def args(self):
        r'''Read-only tuple of markup command arguments.'''
        return self._args

    @property
    def command(self):
        r'''Read-only string of markup command command-name.'''
        return self._command

    @property
    def lilypond_format(self):
        r'''Read-only format of markup command::

            >>> markup_command = markuptools.MarkupCommand('draw-circle', 2.5, 0.1, False)
            >>> markup_command.lilypond_format
            '\\draw-circle #2.5 #0.1 ##f'

        Returns string.
        '''

        return ' '.join(self._format_pieces)

    ### PRIVATE METHODS ###

    def _escape_string(self, string):
        if -1 == string.find(' '):
            return string
        string = repr(string)
        if string.startswith("'") and string.endswith("'"):
            string = string.replace('"', '\"')
            string = '"' + string[1:]
            string = string[:-1] + '"'
        return string

    def _get_format_pieces(self, is_indented=True):
        indent = ''
        if is_indented:
            indent = '\t'
        def recurse(iterable):
            result = []
            for x in iterable:
                if isinstance(x, (list, tuple)):
                    result.append('{')
                    result.extend(recurse(x))
                    result.append('}')
                elif isinstance(x, type(self)):
                    result.extend(x._get_format_pieces(is_indented=is_indented))
                elif isinstance(x, schemetools.Scheme):
                    result.append(x.lilypond_format)
                else:
                    formatted = schemetools.format_scheme_value(x)
                    if isinstance(x, str):
                        result.append(formatted)
                    else:
                        result.append('#{}'.format(formatted))
            return ['{}{}'.format(indent, x) for x in result]
        parts = [r'\%s' % self.command]
        parts.extend(recurse(self.args))
        return parts

    def _get_tools_package_qualified_repr_pieces(self, is_indented=True):
        def recurse(args, is_indented=True):
            indent = '\t'
            if not is_indented:
                indent = ''
            result = []
            for i, arg in enumerate(args):
                if len(args) == 1:
                    comma = ''
                elif 1 < len(args) and i == len(args) - 1:
                    comma = ''    
                else:
                    comma = ','
                if isinstance(arg, AbjadObject):
                    pieces = arg._get_tools_package_qualified_repr_pieces(is_indented=is_indented)
                    for piece in pieces[:-1]:
                        result.append('{}{}'.format(indent, piece))
                    result.append('{}{}{}'.format(indent, pieces[-1], comma))
                elif isinstance(arg, (list, tuple)):
                    result.append('{}['.format(indent))
                    pieces = recurse(arg, is_indented=is_indented)
                    for piece in pieces:
                        result.append('{}{}'.format(indent, piece))
                    result.append('{}]{}'.format(indent, comma))
                else:
                    result.append('{}{!r}{}'.format(indent, arg, comma))
            return result

        indent = '\t'
        if not is_indented:
            indent = ''
        result = []
        result.append('{}('.format(self._tools_package_qualified_class_name))
        result.append('{}{!r},'.format(indent, self.command))
        result.extend(recurse(self.args, is_indented=is_indented))
        result.append('{})'.format(indent))
        return result



from abjad.tools import durationtools


def split_intervals_at_rationals(intervals, offsets):
    '''Split `intervals` at each offset in
    `offsets` ::

        >>> from abjad.tools.timeintervaltools import TimeInterval
        >>> from abjad.tools.timeintervaltools import TimeIntervalTree

    ::

        >>> a = TimeInterval(-1, 3)
        >>> b = TimeInterval(6, 12)
        >>> c = TimeInterval(9, 16)
        >>> tree = TimeIntervalTree([a, b, c])
        >>> timeintervaltools.split_intervals_at_rationals(tree, [1, Fraction(19, 2)])
        TimeIntervalTree([
            TimeInterval(Offset(-1, 1), Offset(1, 1), {}),
            TimeInterval(Offset(1, 1), Offset(3, 1), {}),
            TimeInterval(Offset(6, 1), Offset(19, 2), {}),
            TimeInterval(Offset(9, 1), Offset(19, 2), {}),
            TimeInterval(Offset(19, 2), Offset(12, 1), {}),
            TimeInterval(Offset(19, 2), Offset(16, 1), {})
        ])

    Return TimeIntervalTree.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    offsets = [durationtools.Offset(x) for x in offsets]

    if not tree or not offsets:
        return tree

    for offset in offsets:
        intersecting_intervals = set(tree.find_intervals_intersecting_or_tangent_to_offset(offset))
        if not intersecting_intervals:
            continue
        tangent_intervals = tree.find_intervals_starting_or_stopping_at_offset(offset)
        if tangent_intervals:
            intersecting_intervals = intersecting_intervals.difference(set(tangent_intervals))
        splits = []
        for interval in intersecting_intervals:
            splits.extend(interval.split_at_rationals(offset))
        tree = timeintervaltools.TimeIntervalTree(
            set(tree[:]).difference(intersecting_intervals).union(set(splits)))

    return tree



from abjad.tools import containertools
from abjad.tools import leaftools


def iterate_topmost_masked_tie_chains_and_containers_in_expr(expr):
    r'''Iterate topmost tie chains and containers in `expr`, masked by `expr`:

    ::

        >>> input = "abj: | 2/4 c'4 d'4 ~ |"
        >>> input += "| 4/4 d'4 ~ 2/3 { d'4 d'4 d'4 } d'4 ~ |"
        >>> input += "| 2/4 d'4 e'4 |"
        >>> staff = Staff(input)
        >>> f(staff)
        \new Staff {
            {
                \time 2/4
                c'4
                d'4 ~
            }
            {
                \time 4/4
                d'4 ~
                \times 2/3 {
                    d'4
                    d'4
                    d'4
                }
                d'4 ~
            }
            {
                \time 2/4
                d'4
                e'4
            }
        }

    ::

        >>> for x in tietools.iterate_topmost_masked_tie_chains_and_containers_in_expr(
        ...     staff[0]): x
        ... 
        TieChain(Note("c'4"),)
        TieChain(Note("d'4"),)

    ::

        >>> for x in tietools.iterate_topmost_masked_tie_chains_and_containers_in_expr(
        ...     staff[1]): x
        ... 
        TieChain(Note("d'4"),)
        Tuplet(2/3, [d'4, d'4, d'4])
        TieChain(Note("d'4"),)

    ::

        >>> for x in tietools.iterate_topmost_masked_tie_chains_and_containers_in_expr(
        ...     staff[2]): x
        ... 
        TieChain(Note("d'4"),)
        TieChain(Note("e'4"),)

    Return generator.
    '''

    from abjad.tools import tietools

    last_tie_chain = None
    for x in expr:
        if isinstance(x, leaftools.Leaf):
            tie_chain = tietools.get_tie_chain(x)
            if tie_chain == last_tie_chain:
                continue
            last_tie_chain = tie_chain
            new_tie_chain = []
            index = tie_chain[:].index(x)
            while index < len(tie_chain):
                leaf = tie_chain[index] 
                if leaf.parent is x.parent:
                    new_tie_chain.append(leaf)
                else:
                    break
                index += 1
            yield tietools.TieChain(tuple(new_tie_chain))
        elif isinstance(x, containertools.Container):
            last_tie_chain = None
            yield x



from abjad.tools.lilypondfiletools.NonattributedBlock import NonattributedBlock


class ScoreBlock(NonattributedBlock):
    r'''.. versionadded:: 2.0

    Abjad model of LilyPond input file score block::

        >>> score_block = lilypondfiletools.ScoreBlock()

    ::

        >>> score_block
        ScoreBlock()

    ::

        >>> score_block.append(Staff([]))
        >>> f(score_block)
        \score {
            \new Staff {
            }
        }

    ScoreBlocks does not format when empty, as this generates a 
    parser error in LilyPond::

        >>> score_block = lilypondfiletools.ScoreBlock()
        >>> score_block.lilypond_format == ''
        True

    Return score block.
    '''

    def __init__(self):
        NonattributedBlock.__init__(self)
        self._escaped_name = r'\score'
        self._is_formatted_when_empty = False

    ### PRIVATE PROPERTIES ###

    @property
    def _format_pieces(self):
        from abjad.tools import leaftools
        result = []
        if not len(self):
            if self._is_formatted_when_empty:
                result.append(r'%s {}' % self._escaped_name)
        else:
            result.append(r'%s {' % self._escaped_name)
            if len(self) == 1 and isinstance(self[0], leaftools.Leaf):
                result.append('\t{')
                result.extend(['\t\t' + piece for piece in self[0]._format_pieces])
                result.append('\t}')
            else:
                for x in self:
                    if hasattr(x, '_get_format_pieces'):
                        result.extend(['\t' + piece for piece in x._get_format_pieces()])
                    elif hasattr(x, '_format_pieces'):
                        result.extend(['\t' + piece for piece in x._format_pieces])
                    elif isinstance(x, str):
                        result.append('\t%s' % x)
            result.append('}')
        return result




import subprocess


def spawn_subprocess(command):
    '''.. versionadded:: 2.9

    Spawn subprocess, run `command`, redirect stderr to stdout and print result::

        >>> iotools.spawn_subprocess('echo "hello world"') # doctest: +SKIP
        hello world

    The function is basically a reimplementation of the deprecated ``os.system()``
    using Python's ``subprocess`` module.

    The function provides a type of shell access from the Abjad interpreter.

    Return none.
    '''

    subprocess.call(command, shell=True)
    #proc = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    #for line in proc.stdout.read().splitlines():
    #    print line



from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_LilyPondParser__leaves__Chord_01():
    target = Chord([0, 1, 4], (1, 4))
    parser = LilyPondParser()
    result = parser('{ %s }' % target.lilypond_format)
    assert target.lilypond_format == result[0].lilypond_format and target is not result[0]




def remove_markup_attached_to_component(component):
    r'''.. versionadded:: 2.0

    Remove markup attached to `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> slur = spannertools.SlurSpanner(staff[:])
        >>> markuptools.Markup('foo')(staff[0])
        Markup(('foo',))(c'8)
        >>> markuptools.Markup('bar')(staff[0])
        Markup(('bar',))(c'8)

    ::

        >>> f(staff)
        \new Staff {
            c'8 (
                - \markup {
                    \column
                        {
                            foo
                            bar
                        }
                    }
            d'8
            e'8
            f'8 )
        }

    ::

        >>> markuptools.remove_markup_attached_to_component(staff[0])
        (Markup(('foo',)), Markup(('bar',)))

    ::

        >>> f(staff)
        \new Staff {
            c'8 (
            d'8
            e'8
            f'8 )
        }

    Return tuple of zero or more markup objects.
    '''
    from abjad.tools import markuptools

    # get markup attached to component
    result = markuptools.get_markup_attached_to_component(component)

    # remove markup attached to component
    for mark in result:
        mark()

    # return removed markup
    result = tuple(result)
    return result



import abc
import bisect
import copy
from abjad.tools import datastructuretools
from abjad.tools import sequencetools
from abjad.tools.abctools import AbjadObject


class QSchema(AbjadObject):
    '''The *schema* for a quantization run.

    ``QSchema`` allows for the specification of quantization settings
    diachronically, at any time-step of the quantization process.

    In practice, this provides a means for the composer to change the
    tempo, search-tree, time-signature etc., effectively creating
    a template into which quantized rhythms can be "poured", without
    yet knowing what those rhythms might be, or even how much time the
    ultimate result will take.  Like Abjad's ``ContextMarks``, the
    settings made at any given time-step via a ``QSchema`` instance
    are understood to persist until changed.

    All concrete ``QSchema`` subclasses strongly implement default
    values for all of their parameters.

    `QSchema` is abstract.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_items', '_lookups')

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, *args, **kwargs):
        if 1 == len(args) and isinstance(args[0], type(self)):
            items = copy.copy(args[0].items)

        elif 1 == len(args) and isinstance(args[0], dict):
            items = args[0].items()
            if sequencetools.all_are_pairs_of_types(items, int, dict):
                items = [(x, self.item_klass(**y)) for x, y in items]
            assert sequencetools.all_are_pairs_of_types(items, int, self.item_klass)
            items = dict(items)

        elif sequencetools.all_are_pairs_of_types(args, int, self.item_klass):
            items = dict(args)
            
        elif sequencetools.all_are_pairs_of_types(args, int, dict):
            items = [(x, self.item_klass(**y)) for x, y in args]
            items = dict(items)

        elif all([isinstance(x, self.item_klass) for x in args]):
            items = [(i, x) for i, x in enumerate(args)]
            items = dict(items)

        elif all([isinstance(x, dict) for x in args]):
            items = [(i, self.item_klass(**x)) for i, x in enumerate(args)]
            items = dict(items)

        else:
            raise ValueError

        if items:
            assert 0 <= min(items)

        self._items = datastructuretools.ImmutableDictionary(items)

        self._lookups = self._create_lookups()

    ### SPECIAL METHODS ###

    def __call__(self, duration):
        target_items = []
        idx, current_offset = 0, 0
        while current_offset < duration:
            lookup = self[idx]
            lookup['offset_in_ms'] = current_offset
            target_item = self.target_item_klass(**lookup)
            target_items.append(target_item)
            current_offset += target_item.duration_in_ms
            idx += 1
        return self.target_klass(target_items)

    def __getitem__(self, i):
        assert isinstance(i, int) and 0 <= i
        result = {}
        for field in self._lookups:
            lookup = self._lookups[field].get(i)
            if lookup is not None:
                result[field] = lookup
            else:
                keys = sorted(self._lookups[field].keys())
                idx = bisect.bisect(keys, i)
                if len(keys) == idx:
                    key = keys[-1]
                elif i < keys[idx]:
                    key = keys[idx - 1]
                result[field] = self._lookups[field][key]
        return result

    def __repr__(self):
        if self.items:
            result = ['{}({{'.format(self._tools_package_qualified_class_name)]
            for i, pair in enumerate(sorted(self.items.items())):
                key, value = pair
                itemrepr = value._get_tools_package_qualified_repr_pieces()
                result.append('\t{}: {}'.format(key, itemrepr[0]))
                result.extend(['\t{}'.format(x) for x in itemrepr[1:-1]])
                if i == len(self.items) - 1:
                    result.append('\t{}'.format(itemrepr[-1]))
                else:
                    result.append('\t{},'.format(itemrepr[-1]))
            result.append('\t},')
        else:
            result = ['{}('.format(self._tools_package_qualified_class_name)]
        result.extend(self._get_tools_package_qualified_keyword_argument_repr_pieces())
        result.append('\t)')
        return '\n'.join(result)

    ### READ-ONLY PRIVATE PROPERTIES ###

    @abc.abstractproperty
    def _keyword_argument_names(self):
        raise NotImplemented

    @property
    def _positional_argument_names(self):
        return ('items',)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @abc.abstractproperty
    def item_klass(self):
        '''The schema's item class.'''
        raise NotImplemented

    @property
    def items(self):
        '''The item dictionary.'''
        return self._items

    @property
    def search_tree(self):
        '''The default search tree.'''
        return self._search_tree

    @abc.abstractproperty
    def target_item_klass(self):
        '''The schema's target class' item class.'''
        raise NotImplemented

    @abc.abstractproperty
    def target_klass(self):
        '''The schema's target class.'''
        raise NotImplemented

    @property
    def tempo(self):
        '''The default tempo.'''
        return self._tempo

    ### PRIVATE METHODS ###

    def _create_lookups(self):
        lookups = {}
        fields = self.item_klass._fields
        for field in fields:
            lookups[field] = {0: getattr(self, field)}
            for position, item in self.items.iteritems():
                value = getattr(item, field)
                if value is not None:
                    lookups[field][position] = value
            lookups[field] = datastructuretools.ImmutableDictionary(lookups[field])
        return datastructuretools.ImmutableDictionary(lookups)

    def _get_tools_package_qualified_repr_pieces(self):
        if not len(self.items):
            return ['{}()'.format(self._tools_package_qualified_class_name)]
        result = ['{}({{'.format(self._tools_package_qualified_class_name)]
        for key, value in sorted(self.items.items()):
            value_repr = value._get_tools_package_qualified_repr_pieces()
            result.append('\t{}: {}'.format(key, value_repr[0]))
            result.extend(['\t' + x for x in value_repr[1:-1]])
            result.append('\t{},'.format(value_repr[-1]))
        result.append('\t})')
        return result




from abjad.tools import abctools


class LilyPondDimension(abctools.AbjadObject):
    r'''Abjad model of page dimensions in LilyPond:

    ::

        >>> dimension = lilypondfiletools.LilyPondDimension(2, 'in')

    ::

        >>> f(dimension)
        2.0\in

    Return LilyPondDimension instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_unit', '_value')

    ### INITIALIZER ###

    def __init__(self, value, unit):
        assert 0 <= value
        assert unit in ('cm', 'in', 'mm')
        self._value = float(value)
        self._unit = unit

    ### PRIVATE PROPERTIES ###

    @property
    def _format_pieces(self):
        return [r'{}\{}'.format(self.value, self.unit)]

    ### READ-ONLY PUBLIC PROPERTIES ###
    
    @property
    def lilypond_format(self):
        return '\n'.join(self._format_pieces)

    @property
    def unit(self):
        return self._unit

    @property
    def value(self):
        return self._value



import abc
from abjad.tools.timeintervaltools.TimeIntervalMixin import TimeIntervalMixin


class TimeIntervalAggregateMixin(TimeIntervalMixin):

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta
    __slots__ = ()

    ### PUBLIC PROPERTIES ###

    @abc.abstractproperty
    def earliest_start(self):
        raise NotImplementedError

    @abc.abstractproperty
    def earliest_stop(self):
        raise NotImplementedError

    @abc.abstractproperty
    def intervals(self):
        raise NotImplementedError

    @abc.abstractproperty
    def latest_start(self):
        raise NotImplementedError

    @abc.abstractproperty
    def latest_stop(self):
        raise NotImplementedError

    @property
    def offset_counts(self):
        from collections import Counter
        offsets = []
        for interval in self.intervals:
            offsets.append(interval.start)
            offsets.append(interval.stop)
        return dict(Counter(offsets))

    @property
    def offsets(self):
        offsets = []
        for interval in self.intervals:
            offsets.append(interval.start)
            offsets.append(interval.stop)
        return set(offsets)

    ### PUBLIC METHODS ###

    @abc.abstractmethod
    def find_intervals_intersecting_or_tangent_to_interval(self):
        raise NotImplementedError

    @abc.abstractmethod
    def find_intervals_intersecting_or_tangent_to_offset(self):
        raise NotImplementedError

    @abc.abstractmethod
    def find_intervals_starting_after_offset(self):
        raise NotImplementedError

    @abc.abstractmethod
    def find_intervals_starting_and_stopping_within_interval(self):
        raise NotImplementedError

    @abc.abstractmethod
    def find_intervals_starting_at_offset(self):
        raise NotImplementedError

    @abc.abstractmethod
    def find_intervals_starting_before_offset(self):
        raise NotImplementedError

    @abc.abstractmethod
    def find_intervals_starting_or_stopping_at_offset(self):
        raise NotImplementedError

    @abc.abstractmethod
    def find_intervals_starting_within_interval(self):
        raise NotImplementedError

    @abc.abstractmethod
    def find_intervals_stopping_after_offset(self):
        raise NotImplementedError

    @abc.abstractmethod
    def find_intervals_stopping_at_offset(self):
        raise NotImplementedError

    @abc.abstractmethod
    def find_intervals_stopping_before_offset(self):
        raise NotImplementedError

    @abc.abstractmethod
    def find_intervals_stopping_within_interval(self):
        raise NotImplementedError



from abjad import *


def test_spannertools_report_format_contributions_of_improper_spanners_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    beam = beamtools.BeamSpanner(staff.leaves)
    slur = spannertools.SlurSpanner(staff.leaves)
    trill = spannertools.TrillSpanner(staff)

    r'''
    \new Staff {
        c'8 [ ( \startTrillSpan
        d'8
        e'8
        f'8 ] ) \stopTrillSpan
    }
    '''

    r'''
    BeamSpanner
        _format_right_of_leaf
            [
    SlurSpanner
        _format_right_of_leaf
            (
    TrillSpanner
        _format_right_of_leaf
            \startTrillSpan
    '''

    assert spannertools.report_format_contributions_of_improper_spanners(staff[0]) == 'BeamSpanner\n\t_format_right_of_leaf\n\t\t[\nSlurSpanner\n\t_format_right_of_leaf\n\t\t(\nTrillSpanner\n\t_format_right_of_leaf\n\t\t\\startTrillSpan\n'



from abjad.tools.spannertools.DirectedSpanner.DirectedSpanner import DirectedSpanner


class BeamSpanner(DirectedSpanner):
    r'''Abjad beam spanner::

        >>> staff = Staff("c'8 d'8 e'8 f'8 g'2")

    ::

        >>> f(staff)
        \new Staff {
            c'8
            d'8
            e'8
            f'8
            g'2
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> beamtools.BeamSpanner(staff[:4])
        BeamSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(staff)
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
            g'2
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return beam spanner.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None, direction=None):
        DirectedSpanner.__init__(self, components, direction)

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        DirectedSpanner._copy_keyword_args(self, new)

    def _format_right_of_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            if self.direction is not None:
                result.append('%s [' % self.direction)
            else:
                result.append('[')
        if self._is_my_last_leaf(leaf):
            result.append(']')
        return result



from abjad.tools.rhythmtreetools import RhythmTreeContainer


class QGridContainer(RhythmTreeContainer):
    '''A container in a ``QGrid`` structure:

    ::

        >>> container = quantizationtools.QGridContainer()

    ::

        >>> container
        QGridContainer(
            duration=Duration(1, 1)
            )

    Used internally by ``QGrid``.

    Return ``QGridContainer`` instance.
    '''

    ### CLASS ATTRIBUTES ###

    #__slots__ = ()

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _leaf_klass(self):
        from abjad.tools import quantizationtools
        return quantizationtools.QGridLeaf

    @property
    def _node_klass(self):
        from abjad.tools import quantizationtools
        return (type(self), quantizationtools.QGridLeaf)




from abjad.tools.abctools import AbjadObject


class ContainmentSignature(AbjadObject):
    r'''.. versionadded:: 2.9

    Containment signature of Abjad component::

        >>> score = Score(r"""\context Staff = "CustomStaff" { """
        ...     r"""\context Voice = "CustomVoice" { c' d' e' f' } }""")
        >>> score.name = 'CustomScore'

    ::

        >>> f(score)
        \context Score = "CustomScore" <<
            \context Staff = "CustomStaff" {
                \context Voice = "CustomVoice" {
                    c'4
                    d'4
                    e'4
                    f'4
                }
            }
        >>

    ::

        >>> score.leaves[0].parentage.containment_signature
        ContainmentSignature(Note-..., Voice-'CustomVoice', Staff-..., Score-'CustomScore')

    Used for thread iteration behind the scenes.
    '''

    ### INITIALIZER ###

    def __init__(self):
        self._root = None
        self._root_str = ''
        self._self = None
        self._score = None
        self._staff = None
        self._staff_group = None
        self._voice = None

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        return isinstance(arg, ContainmentSignature) and \
            self._voice == arg._voice and \
            self._staff == arg._staff and \
            self._staff_group == arg._staff_group and \
            self._score == arg._score and \
            self._root == arg._root

    def __ne__(self, arg):
        return not self == arg

    def __repr__(self):
        values = [x[-1] for x in self._parts]
        values = ', '.join(values)
        return '{}({})'.format(self._class_name, values)

    def __str__(self):
        result = []
        for name, value in reversed(self._parts):
            result.append('%10s: %s' % (name, value))
        return '\n'.join(result)

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _parts(self):
        result = []
        attrs = ('self', 'voice', 'staff', 'staff_group', 'score')
        for attr in attrs:
            if getattr(self, '_' + attr, None) is not None:
                result.append((attr, getattr(self, '_' + attr)))
        return result



from abjad.tools import componenttools
from abjad.tools import containertools
from abjad.tools import leaftools


def get_previous_measure_from_component(component):
    '''.. versionadded:: 1.1

    Get previous measure from `component`.

    When `component` is voice, staff or other sequential context,
    and when `component` contains a measure, return last measure
    in `component`. This starts the process of backwards measure iteration. ::

        >>> staff = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 2)
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)
        >>> measuretools.get_previous_measure_from_component(staff)
        Measure(2/8, [e'8, f'8])

    When `component` is voice, staff or other sequential context,
    and when `component` contains no measure,
    raise missing measure error.

    When `component` is a measure and there is a measure immediately
    preceeding `component`, return measure immediately preceeding component. ::

        >>> staff = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 2)
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)
        >>> measuretools.get_previous_measure_from_component(staff[-1])
        Measure(2/8, [c'8, d'8])

    When `component` is a measure and there is no measure immediately
    preceeding `component`, return ``None``. ::

        >>> staff = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 2)
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)
        >>> measuretools.get_previous_measure_from_component(staff[0]) is None
        True

    When `component` is a leaf and there is a measure in the parentage
    of `component`, return the measure in the parentage of `component`. ::

        >>> staff = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 2)
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)
        >>> measuretools.get_previous_measure_from_component(staff.leaves[0])
        Measure(2/8, [c'8, d'8])

    When `component` is a leaf and there is no measure in the parentage
    of `component`, raise missing measure error.

    .. versionchanged:: 2.0
        renamed ``iterate.measure_prev()`` to
        ``measuretools.get_previous_measure_from_component()``.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import measuretools

    if isinstance(component, leaftools.Leaf):
        for parent in componenttools.get_proper_parentage_of_component(component):
            if isinstance(parent, measuretools.Measure):
                return parent
        raise MissingMeasureError
    elif isinstance(component, measuretools.Measure):
        return componenttools.get_nth_namesake_from_component(component, -1)
    elif isinstance(component, containertools.Container):
        return measuretools.get_measure_that_stops_with_container(component)
    elif isinstance(component, (list, tuple)):
        measure_generator = iterationtools.iterate_measures_in_expr(component, reverse=True)
        try:
            measure = measure_generator.next()
            return measure
        except StopIteration:
            raise MissingMeasureError
    else:
        raise TypeError('"%s" is unknown Abjad component.' % component)



from abjad import *


def test_instrumenttools_Flute_storage_format_01():

    flute = instrumenttools.Flute()

    assert flute.storage_format == 'instrumenttools.Flute()'


def test_instrumenttools_Flute_storage_format_02():
    '''Works with explicit instrument name markup.
    '''

    flute = instrumenttools.Flute(instrument_name_markup=markuptools.Markup('Foo'))

    assert flute.storage_format == "instrumenttools.Flute(\n\tinstrument_name_markup=markuptools.Markup((\n\t\t'Foo',\n\t\t))\n\t)"
